﻿ #region Disclaimer/Info
 
 /////////////////////////////////////////////////////////////////////////////////////////////////
 //
 //   File:		TrackBackRepository.cs
 //   Website:		http://dexterblogengine.com/
 //   Authors:		http://dexterblogengine.com/About.ashx
 //   Rev:		1
 //   Created:		19/01/2011
 //   Last edit:		19/01/2011
 //   License:		GNU Library General Public License (LGPL)
 //   File:            TrackBackRepository.cs
 //   For updated news and information please visit http://dexterblogengine.com/
 //   Dexter is hosted to Codeplex at http://dexterblogengine.codeplex.com
 //   For any question contact info@dexterblogengine.com
 //
 ///////////////////////////////////////////////////////////////////////////////////////////////////
 
 #endregion

using System;
using System.Collections.Generic;
using System.Linq;
using Dexter.Core.Data.Repository;
using Dexter.Domain.Model;
using Dexter.Domain.Results;
using Dexter.Logger.Contracts;
using Dexter.Repository.NHibernate.Queries;
using NHibernate;
using Topics.Radical.Validation;
using ISessionFactory = Dexter.Repository.NHibernate.SessionFactory.ISessionFactory;

namespace Dexter.Repository.NHibernate.Repositories {
	/// <summary>
	/// The concrete implementation of <see cref="ITrackBackRepository"/>.
	/// </summary>
	/// <example>The following is an example of initializing 
	///          an <c>TrackBackRepository</c> type:
	///   <code>
	///       // Create the type.
	///       TrackBackRepository ds = new TrackBackRepository;
	/// 
	///       if ( null == ds )
	///           return;
	/// 
	///       return ds.DoSomething( 5 );
	///   </code>
	/// </example>
	public class TrackBackRepository : RepositoryBase <TrackBack> , ITrackBackRepository {
		
		/// <summary>
		/// Initializes a new instance of the <see cref="TrackBackRepository"/> class.
		/// </summary>
		/// <param name="sessionFactory">The session factory.</param>
		/// <param name="logger">The logger.</param>
		public TrackBackRepository ( ISessionFactory sessionFactory , ILogger logger ) : base ( sessionFactory , logger ) {}

		#region Implementation of ITrackBackRepository

		/// <summary>
		/// Adds the specified <c>TrackBack</c> to the repository.
		/// </summary>
		/// <param name="item">The item to save.</param>
		/// <exception cref="ArgumentNullException"> If <paramref name="item"/> is <c>null</c>.</exception>
		public virtual void Save ( TrackBack item ) {
			Ensure.That ( item )
				.Named ( "item" )
				.WithPreview ( ( v , e ) => Logger.Warn ( v.GetFullErrorMessage ( ) , e ) )
				.IsNotNull ( )
				.WithMessage ( "Invalid trackbak." );

			item.ValidateForSave ( );
			Session.SaveOrUpdate ( item );
		}

		/// <summary>
		/// Determines whether is first ping back for the specified post slug.
		/// </summary>
		/// <param name="postSlug">The post slug.</param>
		/// <param name="url">The URL.</param>
		/// <returns>
		/// 	<c>true</c> if is first ping back; otherwise, <c>false</c>.
		/// </returns>
		/// <exception cref="ArgumentNullException"> If <paramref name="postSlug"/> is <c>null</c> or <c>empty</c>.</exception>
		/// <exception cref="ArgumentNullException"> If <paramref name="url"/> is <c>null</c>.</exception>
		public virtual bool IsFirstPingBack ( string postSlug, Uri url ) {
			Ensure.That ( postSlug )
				.Named ( "postSlug" )
				.WithPreview ( ( v , e ) => Logger.Warn ( v.GetFullErrorMessage ( ) , e ) )
				.IsNotNullNorEmpty ( )
				.WithMessage ( "Invalid trackbak." );

			Ensure.That ( url )
				.Named ( "url" )
				.WithPreview ( ( v , e ) => Logger.Warn ( v.GetFullErrorMessage ( ) , e ) )
				.IsNotNull ( )
				.WithMessage ( "Invalid url." );

			return Session.GetNamedQuery ( KnownQueries.TrackBackRepository.IsFirstPingBackFromSlug )
			       	.SetParameter ( "slug" , postSlug )
			       	.SetParameter ( "url" , url )
			       	.UniqueResult <int> ( ) > 0;
		}

		/// <summary>
		/// Determines whether is first ping back for the specified post slug.
		/// </summary>
		/// <param name="postKey">The post key.</param>
		/// <param name="url">The URL.</param>
		/// <returns>
		/// 	<c>true</c> if is first ping back; otherwise, <c>false</c>.
		/// </returns>
		/// <exception cref="ArgumentNullException"> If <paramref name="postKey"/> is lesser than 1.</exception>
		/// <exception cref="ArgumentNullException"> If <paramref name="url"/> is <c>null</c>.</exception>
		public virtual bool IsFirstPingBack ( int postKey, Uri url ) {
			Ensure.That ( postKey )
				.Named ( "postKey" )
				.WithPreview ( ( v , e ) => Logger.Warn ( v.GetFullErrorMessage ( ) , e ) )
				.IsGreaterThen ( 0 )
				.WithMessage ( "Invalid postKey." );

			Ensure.That ( url )
				.Named ( "url" )
				.WithPreview ( ( v , e ) => Logger.Warn ( v.GetFullErrorMessage ( ) , e ) )
				.IsNotNull ( )
				.WithMessage ( "Invalid url." );

			return Session.GetNamedQuery ( KnownQueries.TrackBackRepository.IsFirstPingBackFromPostKey )
			       	.SetParameter ( "postKey" , postKey )
			       	.SetParameter ( "url" , url )
			       	.UniqueResult <int> ( ) > 0;
		}

		/// <summary>
		/// Gets the trackbacks.
		/// </summary>
		/// <param name="pageSize">The number of items to retrieve.</param>
		/// <param name="pageIndex">Index of the page (0 based).</param>
		/// <returns>
		/// An concrete implementation of <see cref="IPagedResult{T}"/>. The result will never be null.
		/// </returns>
		/// <exception cref="ArgumentOutOfRangeException"> If <paramref name="pageIndex"/> is lesser than 0.</exception>
		/// <exception cref="ArgumentOutOfRangeException"> If <paramref name="pageSize"/> is lesser than 1.</exception>
		public virtual IPagedResult<TrackBack> GetTrackBack ( int pageSize, int pageIndex ) {
			return GetTrackBack ( pageIndex , pageSize , true );
		}

		/// <summary>
		/// Gets the trackbacks.
		/// </summary>
		/// <param name="pageSize">The number of items to retrieve.</param>
		/// <param name="pageIndex">Index of the page (0 based).</param>
		/// <param name="enableTracking">if set to <c>true</c> the tracking system of the return object will be enable.</param>
		/// <returns>
		/// An concrete implementation of <see cref="IPagedResult{T}"/>. The result will never be null.
		/// </returns>
		/// <exception cref="ArgumentOutOfRangeException"> If <paramref name="pageIndex"/> is lesser than 0.</exception>
		/// <exception cref="ArgumentOutOfRangeException"> If <paramref name="pageSize"/> is lesser than 1.</exception>
		public virtual IPagedResult<TrackBack> GetTrackBack ( int pageSize, int pageIndex, bool enableTracking ) {
			Ensure.That ( pageIndex )
				.Named ( "pageIndex" )
				.WithPreview ( ( v , e ) => Logger.Warn ( v.GetFullErrorMessage ( ) , e ) )
				.IsGreaterThen ( 0 , Or.Equal )
				.WithMessage ( "Invalid page index." );

			Ensure.That ( pageSize )
				.Named ( "pageSize" )
				.WithPreview ( ( v , e ) => Logger.Warn ( v.GetFullErrorMessage ( ) , e ) )
				.IsGreaterThen ( 0 )
				.WithMessage ( "Invalid page size." );

			IFutureValue <long> total = Session.GetNamedQuery ( KnownQueries.TrackBackRepository.GetTrackBackCount )
				.FutureValue <long> ( );

			IEnumerable <TrackBack> result = Session.GetNamedQuery ( KnownQueries.TrackBackRepository.GetTrackBack )
				.SetFirstResult ( pageIndex * pageSize )
				.SetMaxResults ( pageSize )
				.SetReadOnly ( !enableTracking )
				.Future <TrackBack> ( );

			return new PagedResult<TrackBack>(pageIndex, pageSize, result.ToList(), total.Value);
		}

		/// <summary>
		/// Gets the trackbacks for a specific post.
		/// </summary>
		/// <param name="itemId">The post Id.</param>
		/// <param name="pageSize">The number of items to retrieve.</param>
		/// <param name="pageIndex">Index of the page (0 based).</param>
		/// <returns>
		/// An concrete implementation of <see cref="IPagedResult{T}"/>. The result will never be null.
		/// </returns>
		/// <exception cref="ArgumentOutOfRangeException"> If <paramref name="pageIndex"/> is lesser than 0.</exception>
		/// <exception cref="ArgumentOutOfRangeException"> If <paramref name="pageSize"/> is lesser than 1.</exception>
		/// <exception cref="ArgumentOutOfRangeException"> If <paramref name="itemId"/> is lesser than 1.</exception>
		public virtual IPagedResult<TrackBack> GetTrackBack ( int itemId, int pageSize, int pageIndex ) {
			return GetTrackBack ( itemId , pageIndex , pageSize , true );
		}

		/// <summary>
		/// Gets the trackbacks for a specific post.
		/// </summary>
		/// <param name="itemId">The post Id.</param>
		/// <param name="pageSize">The number of items to retrieve.</param>
		/// <param name="pageIndex">Index of the page (0 based).</param>
		/// <param name="enableTracking">if set to <c>true</c> the tracking system of the return object will be enable.</param>
		/// <returns>
		/// An concrete implementation of <see cref="IPagedResult{T}"/>. The result will never be null.
		/// </returns>
		/// <exception cref="ArgumentOutOfRangeException"> If <paramref name="pageIndex"/> is lesser than 0.</exception>
		/// <exception cref="ArgumentOutOfRangeException"> If <paramref name="pageSize"/> is lesser than 1.</exception>
		/// <exception cref="ArgumentOutOfRangeException"> If <paramref name="itemId"/> is lesser than 1.</exception>
		public virtual IPagedResult<TrackBack> GetTrackBack ( int itemId, int pageSize, int pageIndex, bool enableTracking ) {
			Ensure.That ( pageIndex )
				.Named ( "pageIndex" )
				.WithPreview ( ( v , e ) => Logger.Warn ( v.GetFullErrorMessage ( ) , e ) )
				.IsGreaterThen ( 0 , Or.Equal )
				.WithMessage ( "Invalid page index." );

			Ensure.That ( pageSize )
				.Named ( "pageSize" )
				.WithPreview ( ( v , e ) => Logger.Warn ( v.GetFullErrorMessage ( ) , e ) )
				.IsGreaterThen ( 0 )
				.WithMessage ( "Invalid page size." );

			Ensure.That ( itemId )
				.Named ( "itemID" )
				.WithPreview ( ( v , e ) => Logger.Warn ( v.GetFullErrorMessage ( ) , e ) )
				.IsGreaterThen ( 0 , Or.Equal )
				.WithMessage ( "Invalid item Id." );

			IFutureValue <long> total = Session.GetNamedQuery ( KnownQueries.TrackBackRepository.GetTrackBackForItemByKeyCount )
				.SetParameter ( "key" , itemId )
				.FutureValue <long> ( );

			IEnumerable <TrackBack> result = Session.GetNamedQuery ( KnownQueries.TrackBackRepository.GetTrackBackForItemByKey )
				.SetParameter ( "key" , itemId )
				.SetFirstResult ( pageIndex * pageSize )
				.SetMaxResults ( pageSize )
				.SetReadOnly ( !enableTracking )
				.Future <TrackBack> ( );

			return new PagedResult<TrackBack>(pageIndex, pageSize, result.ToList(), total.Value);
		}

		/// <summary>
		/// Gets the trackbacks for a specific post.
		/// </summary>
		/// <param name="postSlug">The post slug.</param>
		/// <param name="pageSize">The number of items to retrieve.</param>
		/// <param name="pageIndex">Index of the page (0 based).</param>
		/// <returns>
		/// An concrete implementation of <see cref="IPagedResult{T}"/>. The result will never be null.
		/// </returns>
		/// <exception cref="ArgumentOutOfRangeException"> If <paramref name="postSlug"/> is <c>null</c> or <c>empty</c>.</exception>
		/// <exception cref="ArgumentOutOfRangeException"> If <paramref name="pageIndex"/> is lesser than 0.</exception>
		/// <exception cref="ArgumentOutOfRangeException"> If <paramref name="pageSize"/> is lesser than 1.</exception>
		public virtual IPagedResult<TrackBack> GetTrackBack ( string postSlug, int pageSize, int pageIndex ) {
			return GetTrackBack ( postSlug , pageIndex , pageSize , true );
		}

		/// <summary>
		/// Gets the trackbacks for a specific post.
		/// </summary>
		/// <param name="postSlug">The post slug.</param>
		/// <param name="pageSize">The number of items to retrieve.</param>
		/// <param name="pageIndex">Index of the page (0 based).</param>
		/// <param name="enableTracking">if set to <c>true</c> the tracking system of the return object will be enable.</param>
		/// <returns>
		/// An concrete implementation of <see cref="IPagedResult{T}"/>. The result will never be null.
		/// </returns>
		/// <exception cref="ArgumentOutOfRangeException"> If <paramref name="postSlug"/> is <c>null</c> or <c>empty</c>.</exception>
		/// <exception cref="ArgumentOutOfRangeException"> If <paramref name="pageIndex"/> is lesser than 0.</exception>
		/// <exception cref="ArgumentOutOfRangeException"> If <paramref name="pageSize"/> is lesser than 1.</exception>
		public virtual IPagedResult<TrackBack> GetTrackBack ( string postSlug, int pageSize, int pageIndex, bool enableTracking ) {
			Ensure.That ( pageIndex )
				.Named ( "pageIndex" )
				.WithPreview ( ( v , e ) => Logger.Warn ( v.GetFullErrorMessage ( ) , e ) )
				.IsGreaterThen ( 0 , Or.Equal )
				.WithMessage ( "Invalid page index." );

			Ensure.That ( pageSize )
				.Named ( "pageSize" )
				.WithPreview ( ( v , e ) => Logger.Warn ( v.GetFullErrorMessage ( ) , e ) )
				.IsGreaterThen ( 0 )
				.WithMessage ( "Invalid page size." );

			Ensure.That ( postSlug )
				.Named ( "postSlug" )
				.WithPreview ( ( v , e ) => Logger.Warn ( v.GetFullErrorMessage ( ) , e ) )
				.IsNotNullNorEmpty ( )
				.WithMessage ( "Invalid trackbak." );

			IFutureValue <long> total = Session.GetNamedQuery ( KnownQueries.TrackBackRepository.GetTrackBackForItemBySlugCount )
				.SetParameter ( "slug" , postSlug )
				.FutureValue <long> ( );

			IEnumerable <TrackBack> result = Session.GetNamedQuery ( KnownQueries.TrackBackRepository.GetTrackBackForItemBySlug )
				.SetParameter ( "slug" , postSlug )
				.SetFirstResult ( pageIndex * pageSize )
				.SetMaxResults ( pageSize )
				.SetReadOnly ( !enableTracking )
				.Future <TrackBack> ( );

			return new PagedResult<TrackBack>(pageIndex, pageSize, result.ToList(), total.Value);
		}

		#endregion
	}
}
