﻿ #region Disclaimer/Info
 
 /////////////////////////////////////////////////////////////////////////////////////////////////
 //
 //   File:		CommentNotificationService.cs
 //   Website:		http://dexterblogengine.com/
 //   Authors:		http://dexterblogengine.com/About.ashx
 //   Rev:		1
 //   Created:		19/01/2011
 //   Last edit:		19/01/2011
 //   License:		GNU Library General Public License (LGPL)
 // 
 //   For updated news and information please visit http://dexterblogengine.com/
 //   Dexter is hosted to Codeplex at http://dexterblogengine.codeplex.com
 //   For any question contact info@dexterblogengine.com
 //
 ///////////////////////////////////////////////////////////////////////////////////////////////////
 
 #endregion

using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Net.Mail;
using System.Web.Security;
using Dexter.Core.EventArgs;
using Dexter.Domain.Model;
using Dexter.Logger.Contracts;
using Dexter.Services;

namespace Dexter.Services.Implementation {
	/// <summary>
	/// 	Implementation of <see cref = "ICommentNotificationService" />.
	/// </summary>
	public class CommentNotificationService : ICommentNotificationService {
		readonly IConfigurationService configurationService;
		readonly IEmailService emailService;
		readonly IEmailTemplateService emailTemplateService;
		readonly ILogger logger;
		readonly IUrlBuilder urlBuilder;

		/// <summary>
		/// 	Initializes a new instance of the <see cref = "CommentNotificationService" /> class.
		/// </summary>
		/// <param name = "commentService">The comment service.</param>
		/// <param name = "emailService">The email service.</param>
		/// <param name = "configurationService">The configuration service.</param>
		/// <param name = "emailTemplateService">The email template service.</param>
		/// <param name = "logger">The logger.</param>
		/// <param name = "urlBuilder">The URL builder.</param>
		public CommentNotificationService ( ICommentService commentService , IEmailService emailService , IConfigurationService configurationService , IEmailTemplateService emailTemplateService , ILogger logger , IUrlBuilder urlBuilder ) {
			this.urlBuilder = urlBuilder;
			this.logger = logger;
			this.emailService = emailService;
			this.configurationService = configurationService;
			this.emailTemplateService = emailTemplateService;

			commentService.CommentAdded += CommentServiceCommentAdded;
		}

		void CommentServiceCommentAdded ( object sender , GenericEventArgs <Comment> e ) {
			var configuration = configurationService.Configuration;

			if ( !configuration.NotificationConfiguration.NotifyNewComment ) {
				logger.Debug ( "The comment notification service is disabled." );
				return;
			}

			IEnumerable <EmailMessage> emails;

			switch ( configuration.CommentConfiguration.ModerationType ) {
				case ModerationType.First:
					emails = GetModerationNotificationEmail ( configuration , e.Objects );
					break;
				default:
					emails = GetNewCommentNotificationEmail ( configuration , e.Objects );
					break;
			}

			if ( emails == null || emails.Count ( ) < 1 ) {
				return;
			}

			foreach ( var email in emails ) {
				emailService.Queue ( email );
			}
		}

		IEnumerable <EmailMessage> GetModerationNotificationEmail ( SiteConfiguration configuration , Comment comment ) {
			const string TEMPLATE_NAME = "dexter.comment.newcommentmoderation";

			var template = emailTemplateService.GetTemplate ( TEMPLATE_NAME );

			if ( template == null ) {
				logger.WarnFormat ( "Template '{0}' not found." , TEMPLATE_NAME );
				return null;
			}

			IDictionary <string , string> values = new Dictionary <string , string> {
				{"CommentMessage" , comment.Message} ,
				{"PostUrl" , urlBuilder.Post.Detail ( comment.Item )} ,
				{"CommentApproveUrl" , urlBuilder.Comment.Approve ( comment )} ,
				{"CommentDeleteUrl" , urlBuilder.Comment.Delete ( comment )} ,
				{"CommentMarkAsSpamUrl" , urlBuilder.Comment.MarkAsSpam ( comment )} ,
				{"CommentUsername" , comment.Name} ,
				{"PostTitle" , comment.Item.Title}
			};

			string subject = template.GetFormatedSubject ( values );
			string body = template.GetFormatedBody ( values );

			var currentUser = Membership.GetUser ( comment.Item.Username );

			if ( currentUser == null ) {
				logger.Debug ( "There is a new comment for a post without user. Maybe the user is deleted" );
				currentUser = Membership.GetUser ( configuration.PrincipalAuthor );
			}

			var email = new MailAddress ( currentUser.Email , currentUser.UserName );

			var resultEmail = EmailMessage.CreateNewEmailMessage ( configuration.NotificationConfiguration.SenderEmail , email , subject , body );

			return new [] {resultEmail};
		}

		IEnumerable <EmailMessage> GetNewCommentNotificationEmail ( SiteConfiguration configuration , Comment comment ) {
			const string TEMPLATE_NAME = "dexter.comment.newcomment";

			var template = emailTemplateService.GetTemplate ( TEMPLATE_NAME );

			if ( template == null ) {
				logger.WarnFormat ( "Template '{0}' not found." , TEMPLATE_NAME );
				return null;
			}

			IDictionary <string , string> values = new Dictionary <string , string> {
				{"CommentUsername" , comment.Name} ,
				{"PostTitle" , comment.Item.Title} ,
				{"CommentMessage" , comment.Message} ,
				{"PostUrl" , urlBuilder.Post.FeedBack ( comment.Item )}
			};

			string subject = template.GetFormatedSubject ( values );
			string body = template.GetFormatedBody ( values );

			List <MailAddress> notificationComments = comment.Item.Comments.ToFlat ( x => x.Comments )
				.Where ( c => c.Notify && c.Email != comment.Email )
				.GroupBy ( x => x.Email )
				.Select ( c => c.Key )
				.ToList ( );

			IList <EmailMessage> result = new Collection <EmailMessage> ( );

			foreach ( MailAddress email in notificationComments ) {
				result.Add ( EmailMessage.CreateNewEmailMessage ( configuration.NotificationConfiguration.SenderEmail , email , subject , body ) );
			}

			return result;
		}
	}
}
