﻿using System;
using Moq;
using NUnit.Framework;
using SharpTestsEx;

namespace Dexter.Dependency.Test {
	[TestFixture]
	public class DexterContainerTest {
		Mock<IDexterContainer> mockContainer;

		[TestFixtureSetUp]
		public void TestStartUp() {
			 mockContainer =new Mock <IDexterContainer> ( );
		}

		[TestFixtureTearDown]
		public void TestComplete() {
			mockContainer = null;
		}

		[Test]
		public void SetCurrent_ShouldChangeTheCurrentContainer() {
			DexterContainer.SetCurrent ( mockContainer.Object );

			DexterContainer.Engine.Should ( ).Be.EqualTo ( mockContainer );
		}

		[Test]
		public void Configure_ShouldCallConfigureFromMockEngine() {
			DexterContainer.SetCurrent ( mockContainer.Object );

			DexterContainer.Configure ( "myConfig" );

			mockContainer.Verify(x => x.Configure("myConfig"), Times.Once());
		}

		[Test]
		public void ResolveByType_WithNullType_ShouldThrowArgumenNullException ( ) {

			Executing.This ( ( ) => DexterContainer.Resolve ( null ) )
				.Should ( )
				.Throw <ArgumentNullException> ( );
		}

		[Test]
		public void ResolveByTypeAndKey_WithNullType_ShouldThrowArgumenNullException ( ) {

			Executing.This ( ( ) => DexterContainer.Resolve ( null, "myKey" ) )
				.Should ( )
				.Throw<ArgumentNullException> ( );
		}

		[Test]
		public void ResolveByTypeAndKey_WithValidTypeAndNullKey_ShouldThrowArgumenNullException ( ) {

			Executing.This ( ( ) => DexterContainer.Resolve ( typeof ( DexterContainer ), null ) )
				.Should ( )
				.Throw<ArgumentNullException> ( );
		}

		[Test]
		public void ResolveByTypeAndKey_WithValidTypeAndEmptyKey_ShouldThrowArgumenNullException ( ) {

			Executing.This ( ( ) => DexterContainer.Resolve ( typeof ( DexterContainerTest ), "" ) )
				.Should ( )
				.Throw<ArgumentException> ( );
		}

		[Test]
		public void ResolveByTypeAndKey_WithValidData_ShouldInvokeMockMethod() {
			DexterContainer.SetCurrent ( mockContainer.Object );

			DexterContainer.Resolve ( typeof ( DexterContainerTest ) , "MyKey" );

			mockContainer.Verify(x => x.Resolve(typeof(DexterContainerTest), "MyKey"), Times.Once());
		}

		[Test]
		public void ResolveByGenerics_ShouldInvokeMockMethod() {
			DexterContainer.SetCurrent(mockContainer.Object);

			DexterContainer.Resolve <DexterContainerTest> ( );

			mockContainer.Verify(x => x.Resolve<DexterContainerTest>(), Times.Once());
		}

		[Test]
		public void ResolveByGenericsAndKey_WithValidTypeAndNullKey_ShouldThrowArgumenNullException ( ) {

			Executing.This ( ( ) => DexterContainer.Resolve<DexterContainerTest> ( null ) )
				.Should ( )
				.Throw<ArgumentNullException> ( );
		}

		[Test]
		public void ResolveByGenericsAndKey_WithValidTypeAndEmptyKey_ShouldThrowArgumenNullException ( ) {

			Executing.This ( ( ) => DexterContainer.Resolve<DexterContainerTest> ( "" ) )
				.Should ( )
				.Throw<ArgumentException> ( );
		}

		[Test]
		public void RegisterByKeyAndTypeAndTypeAndLifeCycleEnum_WithNullKey_ShouldThrowArgumentNullException() {

			Executing.This ( ( ) => DexterContainer.Register ( null , typeof ( DexterContainerTest ) , typeof ( DexterContainerTest ) , LifeCycle.Singleton ) )
				.Should ( )
				.Throw <ArgumentNullException> ( );
		}

		[Test]
		public void RegisterByKeyAndTypeAndTypeAndLifeCycleEnum_WithEmptyKey_ShouldThrowArgumentNullException ( ) {

			Executing.This ( ( ) => DexterContainer.Register ( "", typeof ( DexterContainerTest ), typeof ( DexterContainerTest ), LifeCycle.Singleton ) )
				.Should ( )
				.Throw<ArgumentException> ( );
		}

		[Test]
		public void RegisterByKeyAndTypeAndTypeAndLifeCycleEnum_WithNullType_ShouldThrowArgumentNullException ( ) {

			Executing.This ( ( ) => DexterContainer.Register ( "MyKey", null, typeof ( DexterContainerTest ), LifeCycle.Singleton ) )
				.Should ( )
				.Throw<ArgumentNullException> ( );
		}

		[Test]
		public void RegisterByKeyAndTypeAndTypeAndLifeCycleEnum_WithNullImplementedType_ShouldThrowArgumentNullException ( ) {

			Executing.This ( ( ) => DexterContainer.Register ( "MyKey", typeof ( DexterContainerTest ), null, LifeCycle.Singleton ) )
				.Should ( )
				.Throw<ArgumentNullException> ( );
		}

		[Test]
		public void RegisterByKeyAndTypeAndTypeAndLifeCycleEnum_WithValidData_ShouldInvokeMockMethod ( ) {
			DexterContainer.SetCurrent(mockContainer.Object);

			DexterContainer.Register ( "MyKey,", typeof ( DexterContainerTest ), typeof ( DexterContainerTest ), LifeCycle.Singleton );

			mockContainer.Verify(x => x.Register("MyKey,", typeof (DexterContainerTest), typeof (DexterContainerTest), LifeCycle.Singleton), Times.Once());
		}

		[Test]
		public void RegisterByGenericsByKeyAndLifeCycleEnum_WithNullKey_ShouldThrowArgumentNullException ( ) {

			Executing.This ( ( ) => DexterContainer.Register<DexterContainerTest, DexterContainerTest> ( null,  LifeCycle.Singleton ) )
				.Should ( )
				.Throw<ArgumentNullException> ( );
		}

		[Test]
		public void RegisterByGenericsByKeyAndLifeCycleEnum_WithEmptyKey_ShouldThrowArgumentNullException ( ) {

			Executing.This ( ( ) => DexterContainer.Register<DexterContainerTest, DexterContainerTest> ( "", LifeCycle.Singleton ) )
				.Should ( )
				.Throw<ArgumentException> ( );
		}

		[Test]
		public void RegisterByGenericsByKeyAndLifeCycleEnum_WithValidData_ShouldInvokeMockMethod ( ) {
			DexterContainer.SetCurrent ( mockContainer.Object );

			DexterContainer.Register<DexterContainerTest, DexterContainerTest> ( "MyKey,", LifeCycle.Singleton );

			mockContainer.Verify(x => x.Register<DexterContainerTest, DexterContainerTest>("MyKey,", LifeCycle.Singleton), Times.Once());
		}

		[Test]
		public void RegisterComponentsByBaseClass_WithNullAssembly_ShouldThrowArgumentNullException ( ) {
			Executing.This ( ( ) => DexterContainer.RegisterComponentsByBaseClass<DexterContainerTest> ( null, LifeCycle.Singleton ) )
				.Should ( )
				.Throw<ArgumentNullException> ( );
		}

		[Test]
		public void RegisterComponentsByBaseClass_WithValidData_ShouldInvokeMockMethod ( ) {
			DexterContainer.SetCurrent ( mockContainer.Object );

			DexterContainer.RegisterComponentsByBaseClass<DexterContainerTest> ( this.GetType ( ).Assembly, LifeCycle.Singleton );

			mockContainer.Verify(x => x.RegisterComponentsByBaseClass<DexterContainerTest> ( this.GetType ( ).Assembly, LifeCycle.Singleton ), Times.Once());
		}

		[Test]
		public void ResolveAll_WithValidData_ShouldInvokeMockMethod ( ) {
			DexterContainer.SetCurrent ( mockContainer.Object );

			DexterContainer.ResolveAll<DexterContainerTest>();

			mockContainer.Verify(x => x.ResolveAll<DexterContainerTest>(), Times.Once());
		}

		[Test]
		public void ResolveAllByType_WithNullType_ShouldThrowArgumentNullException ( ) {
			Executing.This ( ( ) => DexterContainer.ResolveAll ( null ) )
				.Should ( )
				.Throw<ArgumentNullException> ( );
		}

		[Test]
		public void ResolveAllByType_WithValidData_ShouldInvokeMockMethod ( ) {
			DexterContainer.SetCurrent ( mockContainer.Object );

			DexterContainer.ResolveAll ( typeof ( DexterContainerTest ) );

			mockContainer.Verify(x => x.ResolveAll(typeof(DexterContainerTest)), Times.Once());
		}
	}
}