﻿#region Disclaimer/Info

/////////////////////////////////////////////////////////////////////////////////////////////////
//
//   File:		UserTest.cs
//   Website:		http://dexterblogengine.com/
//   Authors:		http://dexterblogengine.com/About.ashx
//   Rev:		1
//   Created:		22/11/2010
//   Last edit:		19/01/2011
//   License:		GNU Library General Public License (LGPL)
//   File:            UserTest.cs
//   For updated news and information please visit http://dexterblogengine.com/
//   Dexter is hosted to Codeplex at http://dexterblogengine.codeplex.com
//   For any question contact info@dexterblogengine.com
//
///////////////////////////////////////////////////////////////////////////////////////////////////

#endregion

using System;
using System.Security.Cryptography;
using System.Text;
using Dexter.Domain.Model;
using NUnit.Framework;
using SharpTestsEx;

namespace Dexter.Domain.Test.Model {
	[TestFixture]
	public class UserTest {
		[Test]
		public void VerifyCryptoAgility ( ) {
			User sut = new User ( "imperugo" , HashProvider.SHA1 , "password" , EncryptDataFormat.Hashed );
			sut.ChangePassword ( "newpwd" );

			String currentPassword = sut.Password;

			sut.VerifyPassword ( "newpwd" , HashProvider.SHA512 ).Should ( ).Be.True ( );
			sut.HashProvider.Should ( ).Be.EqualTo ( HashProvider.SHA512 );
			currentPassword.Should ( ).Not.Be.EqualTo ( sut.Password );
			//Issue a standard login verification
			sut.VerifyPassword ( "newpwd" ).Should ( ).Be.True ( );
		}

		[Test]
		public void VerifyEncryptedPassword ( ) {
			User sut = new User ( "imperugo" , HashProvider.SHA1 , "password" , EncryptDataFormat.Encrypted );
			sut.Password.Should ( )
				.Not
				.Be
				.EqualTo ( "password" );
		}

		[Test]
		public void VerifyGenerationOfSalt ( ) {
			User sut = new User ( "imperugo" , HashProvider.SHA1 , "password" , EncryptDataFormat.Hashed );
			var value = sut.GenerateSalt ( );
			value.Length.Should ( ).Be.EqualTo ( 24 );
		}

		[Test]
		public void VerifyLoginWithClearPassword ( ) {
			User sut = new User ( "imperugo" , HashProvider.SHA1 , "password" , EncryptDataFormat.Clear );
			sut.ChangePassword ( "newpwd" );
			sut.VerifyPassword ( "newpwd" ).Should ( ).Be.True ( );
		}

		[Test]
		public void VerifyLoginWithHashedPassword ( ) {
			User sut = new User ( "imperugo" , HashProvider.SHA1 , "password" , EncryptDataFormat.Hashed );
			sut.ChangePassword ( "newpwd" );
			sut.VerifyPassword ( "newpwd" ).Should ( ).Be.True ( );
		}

		[Test]
		public void VerifyLoginWithHashedPasswordAndDigest ( ) {
			User sut = new User ( "imperugo" , HashProvider.SHA1 , "password" , EncryptDataFormat.Hashed );
			sut.ChangePassword ( "newpwd" );

			//When we have a digest authentication we have a base64 hash of something calculated by
			//Password + hashtoken and we should verify that this is good NOW WE SIMULATE THE CLIENT

			//1) the client received a token from a previous call to the system
			RNGCryptoServiceProvider provider = new RNGCryptoServiceProvider ( );
			byte[] binToken = new byte[16];
			provider.GetBytes ( binToken );
			String token = Convert.ToBase64String ( binToken );

			//2)the client use the token along with the clear textpassword and salt to create the
			//authentication token
			const string PASSWORD = "newpwd";
			const string HASH_PROVIDER = "SHA1";
			string salt = sut.PasswordSalt;

			//This is code that client will use.
			String authToken;
			using ( HashAlgorithm hash = HashAlgorithm.Create ( HASH_PROVIDER ) ) {
				Byte[] bpassword = Encoding.UTF8.GetBytes ( PASSWORD );
				Byte[] bsalt = Convert.FromBase64String ( salt );
				byte[] dst = new byte[bpassword.Length + bsalt.Length];
				Buffer.BlockCopy ( bpassword , 0 , dst , 0 , bpassword.Length );
				Buffer.BlockCopy ( bsalt , 0 , dst , bpassword.Length , bsalt.Length );
				Byte[] serverPassword = hash.ComputeHash ( dst );
				//now the client has the same password that stored in database we need to add token

				Byte[] btoken = Convert.FromBase64String ( token );
				byte[] binaryAuthToken = new byte[btoken.Length + serverPassword.Length];
				Buffer.BlockCopy ( serverPassword , 0 , binaryAuthToken , 0 , serverPassword.Length );
				Buffer.BlockCopy ( btoken , 0 , binaryAuthToken , serverPassword.Length , btoken.Length );

				Byte[] binaryAuthTokenHashed = hash.ComputeHash ( binaryAuthToken );
				authToken = Convert.ToBase64String ( binaryAuthTokenHashed );
			}

			//now we ask to verify if the authentication token is good.
			sut.VerifyPasswordWithDigest ( token , authToken ).Should ( ).Be.True ( );
		}

		[Test]
		public void VerifyPasswordDecryption ( ) {
			User sut = new User ( "imperugo" , HashProvider.SHA1 , "password" , EncryptDataFormat.Encrypted );
			sut.UnencodePassword ( )
				.Should ( )
				.Be
				.EqualTo ( "password" );
		}

		[Test]
		public void VerifySetAPassword ( ) {
			User sut = new User ( "imperugo" , HashProvider.SHA1 , "password" , EncryptDataFormat.Hashed );
			sut.ChangePassword ( "newpwd" );
			sut.PasswordSalt.Length.Should ( ).Be.EqualTo ( 24 );
			sut.Password.Should ( ).Not.Be.Null ( );
		}

		[Test]
		public void VerifySetAPasswordInClearFormat ( ) {
			User sut = new User ( "imperugo" , HashProvider.SHA1 , "password" , EncryptDataFormat.Clear );
			sut.ChangePassword ( "newpwd" );
			sut.Password.Should ( ).Be.EqualTo ( "newpwd" );
		}
	}
}