﻿using System;
using System.IO;
using System.Linq;
using Dexter.Caching;
using Dexter.Localization.Exception;
using Dexter.Storage;
using NUnit.Framework;
using Rhino.Mocks;
using SharpTestsEx;

namespace Dexter.Localization.Po.Test {
	[TestFixture]
	public class LocalizationProviderTest {
		#region GetLocalizedStringTest Method

		[Test]
		public void GetLocalizedStringTest_WithNullModuleName_ShouldFindCoreModule ( ) {
			var mockStorage = MockRepository.GenerateMock <IStorageProvider> ( );
			var stubCache = MockRepository.GenerateStub <ICacheProvider> ( );

			stubCache.Stub ( x => x.Get ( null ) ).IgnoreArguments ( ).Return ( null );
			stubCache.Stub ( x => x.Get ( null , null ) ).IgnoreArguments ( ).Return ( null );

			var sut = new LocalizationProvider ( mockStorage , stubCache );

			try {
				sut.GetLocalizedString ( null , "myScope" , "myText" , "en-US" );
			}
			catch {
				//note: for this test is not important the rest of execution
			}

			string aspectedCall = "Localization/en-US/core.po";

			mockStorage.AssertWasCalled ( x => x.GetFile ( aspectedCall ) );
		}

		[Test]
		public void GetLocalizedStringTest_WithValidModuleName_ShouldFindPassedModuleFile ( ) {
			var mockStorage = MockRepository.GenerateMock <IStorageProvider> ( );
			var stubCache = MockRepository.GenerateStub <ICacheProvider> ( );

			stubCache.Stub ( x => x.Get ( null ) ).IgnoreArguments ( ).Return ( null );
			stubCache.Stub ( x => x.Get ( null , null ) ).IgnoreArguments ( ).Return ( null );

			var sut = new LocalizationProvider ( mockStorage , stubCache );

			sut.GetLocalizedString ( "myModule" , "myScope" , "myText" , "en-US" );

			string aspectedCall = "Localization/en-US/myModule.po";

			mockStorage.AssertWasCalled ( x => x.GetFile ( aspectedCall ) );
		}

		[Test]
		public void GetLocalizedStringTest_WithEmptyText_ShouldThrowArgumentException ( ) {
			var stubStorage = MockRepository.GenerateStub <IStorageProvider> ( );
			var stubCache = MockRepository.GenerateStub <ICacheProvider> ( );

			var sut = new LocalizationProvider ( stubStorage , stubCache );

			Executing.This ( ( ) => sut.GetLocalizedString ( "myModule" , "myScope" , "" , "en-US" ) )
				.Should ( )
				.Throw <ArgumentException> ( );
		}

		[Test]
		public void GetLocalizedStringTest_WithNullText_ShouldThrowArgumentNullException ( ) {
			var stubStorage = MockRepository.GenerateStub <IStorageProvider> ( );
			var stubCache = MockRepository.GenerateStub <ICacheProvider> ( );

			var sut = new LocalizationProvider ( stubStorage , stubCache );

			Executing.This ( ( ) => sut.GetLocalizedString ( "myModule" , "myScope" , null , "en-US" ) )
				.Should ( )
				.Throw <ArgumentNullException> ( );
		}

		[Test]
		public void GetLocalizedStringTest_WithEmptyCulture_ShouldThrowArgumentException ( ) {
			var stubStorage = MockRepository.GenerateStub <IStorageProvider> ( );
			var stubCache = MockRepository.GenerateStub <ICacheProvider> ( );

			var sut = new LocalizationProvider ( stubStorage , stubCache );

			Executing.This ( ( ) => sut.GetLocalizedString ( "myModule" , "myScope" , "myText" , "" ) )
				.Should ( )
				.Throw <ArgumentException> ( );
		}

		[Test]
		public void GetLocalizedStringTest_WithNullCulture_ShouldThrowArgumentNullException ( ) {
			var stubStorage = MockRepository.GenerateStub <IStorageProvider> ( );
			var stubCache = MockRepository.GenerateStub <ICacheProvider> ( );

			var sut = new LocalizationProvider ( stubStorage , stubCache );

			Executing.This ( ( ) => sut.GetLocalizedString ( "myModule" , "myScope" , "myText" , null ) )
				.Should ( )
				.Throw <ArgumentNullException> ( );
		}

		[Test]
		public void GetLocalizedStringTest_WithValidData_ShouldNotThrowException() {
			var stubStorage = MockRepository.GenerateStub<IStorageProvider> ( );
			var stubCache = MockRepository.GenerateStub<ICacheProvider> ( );
			var stubStorageFile = MockRepository.GenerateStub<IStorageFile> ( );

			stubCache.Stub ( x => x.Get ( null ) ).IgnoreArguments ( ).Return ( null );
			stubCache.Stub ( x => x.Get ( null, null ) ).IgnoreArguments ( ).Return ( null );

			stubStorage.Stub ( x => x.GetFile ( null ) ).IgnoreArguments ( ).Return ( stubStorageFile );

			var filePath = AppDomain.CurrentDomain.BaseDirectory + "\\Resources\\en-US\\TestModule.po";

			using ( var sr = new FileStream ( filePath, FileMode.Open, FileAccess.Read ) ) {
				stubStorageFile.Stub ( x => x.OpenRead ( ) ).Return ( sr );

				var sut = new LocalizationProvider ( stubStorage, stubCache );

				var result = sut.GetLocalizedString ( "TestModule", "My Scope", "My message Id", "en-US" );

				result.Should ( ).Not.Be.Null ( );
				result.Text.Should ( ).Be.EqualTo ( "My message string" );
			}
		}

		[Test]
		public void GetLocalizedStringTest_WithValidDataAndWithoutScope_ShouldNotThrowException ( ) {
			var stubStorage = MockRepository.GenerateStub<IStorageProvider> ( );
			var stubCache = MockRepository.GenerateStub<ICacheProvider> ( );
			var stubStorageFile = MockRepository.GenerateStub<IStorageFile> ( );

			stubCache.Stub ( x => x.Get ( null ) ).IgnoreArguments ( ).Return ( null );
			stubCache.Stub ( x => x.Get ( null, null ) ).IgnoreArguments ( ).Return ( null );

			stubStorage.Stub ( x => x.GetFile ( null ) ).IgnoreArguments ( ).Return ( stubStorageFile );

			var filePath = AppDomain.CurrentDomain.BaseDirectory + "\\Resources\\en-US\\TestModule.po";

			using (var sr = new FileStream ( filePath, FileMode.Open, FileAccess.Read )) {
				stubStorageFile.Stub ( x => x.OpenRead ( ) ).Return ( sr );

				var sut = new LocalizationProvider ( stubStorage, stubCache );

				var result = sut.GetLocalizedString ( "TestModule", null, "My message Id without scope", "en-US" );
				
				result.Should ( ).Not.Be.Null ( );
				result.Text.Should ( ).Be.EqualTo ( "My message string without scope" );
			}
		}

		#endregion

		#region GetAllLocalizedStringForSpecifiedScope Method

		[Test]
		public void GetAllLocalizedStringForSpecifiedScope_WithEmptyScope_ShouldThrowArgumentException() {
			var stubStorage = MockRepository.GenerateStub<IStorageProvider> ( );
			var stubCache = MockRepository.GenerateStub<ICacheProvider> ( );

			var sut = new LocalizationProvider ( stubStorage, stubCache );

			Executing.This ( ( ) => sut.GetAllLocalizedStringForSpecifiedScope ( "myModule", "", "en-US" ) )
				.Should ( )
				.Throw<ArgumentException> ( );
		}

		[Test]
		public void GetAllLocalizedStringForSpecifiedScope_WithNullScope_ShouldThrowArgumentException ( ) {
			var stubStorage = MockRepository.GenerateStub<IStorageProvider> ( );
			var stubCache = MockRepository.GenerateStub<ICacheProvider> ( );

			var sut = new LocalizationProvider ( stubStorage, stubCache );

			Executing.This ( ( ) => sut.GetAllLocalizedStringForSpecifiedScope ( "myModule", null, "en-US" ) )
				.Should ( )
				.Throw<ArgumentNullException> ( );
		}

		[Test]
		public void GetAllLocalizedStringForSpecifiedScope_WithNullModuleName_ShouldFindCoreModule ( ) {
			var mockStorage = MockRepository.GenerateMock<IStorageProvider> ( );
			var stubCache = MockRepository.GenerateStub<ICacheProvider> ( );

			stubCache.Stub ( x => x.Get ( null ) ).IgnoreArguments ( ).Return ( null );
			stubCache.Stub ( x => x.Get ( null, null ) ).IgnoreArguments ( ).Return ( null );

			var sut = new LocalizationProvider ( mockStorage, stubCache );

			try {
				sut.GetAllLocalizedStringForSpecifiedScope ( null, "myScope", "en-US" );
			}
			catch {
				//note: for this test is not important the rest of execution
			}

			string aspectedCall = "Localization/en-US/core.po";

			mockStorage.AssertWasCalled ( x => x.GetFile ( aspectedCall ) );
		}

		[Test]
		public void GetAllLocalizedStringForSpecifiedScope_WithValidScope_ShouldReturnValidData() {
			var stubStorage = MockRepository.GenerateStub<IStorageProvider> ( );
			var stubCache = MockRepository.GenerateStub<ICacheProvider> ( );
			var stubStorageFile = MockRepository.GenerateStub<IStorageFile> ( );

			stubCache.Stub ( x => x.Get ( null ) ).IgnoreArguments ( ).Return ( null );
			stubCache.Stub ( x => x.Get ( null, null ) ).IgnoreArguments ( ).Return ( null );

			stubStorage.Stub ( x => x.GetFile ( null ) ).IgnoreArguments ( ).Return ( stubStorageFile );

			var filePath = AppDomain.CurrentDomain.BaseDirectory + "\\Resources\\en-US\\TestModule.po";

			using (var sr = new FileStream ( filePath, FileMode.Open, FileAccess.Read )) {
				stubStorageFile.Stub ( x => x.OpenRead ( ) ).Return ( sr );

				var sut = new LocalizationProvider ( stubStorage, stubCache );

				var result = sut.GetAllLocalizedStringForSpecifiedScope ( "My Scope" , "en-US" );

				result.Should ( ).Not.Be.Null ( );
				result.Count ( ).Should ( ).Be.EqualTo ( 3 );
				result.ToList ( )[0].MsgId.Should ( ).Be.EqualTo ( "my message id" );
				result.ToList ( )[0].Text.Should ( ).Be.EqualTo ( "My message string" );
				result.ToList ( )[1].MsgId.Should ( ).Be.EqualTo ( "my message id2" );
				result.ToList ( )[1].Text.Should ( ).Be.EqualTo ( "My message string2" );
				result.ToList ( )[2].MsgId.Should ( ).Be.EqualTo ( "my message id3" );
				result.ToList ( )[2].Text.Should ( ).Be.EqualTo ( "My message string3" );
			}
		}

		#endregion

		#region SaveModuleMethod

		[Test]
		public void SaveModule_WithEmptyCultureName_ShouldThrowArgumentException() {
			var stubStorage = MockRepository.GenerateStub<IStorageProvider> ( );
			var stubCache = MockRepository.GenerateStub<ICacheProvider> ( );

			var sut = new LocalizationProvider ( stubStorage, stubCache );

			using (var readStream = Helper.GetTestModuleStream ( )) {
				Executing.This ( ( ) => sut.SaveModule ( "", "Module1", readStream ) )
					.Should ( )
					.Throw<ArgumentException> ( );
			}
		}

		[Test]
		public void SaveModule_WithNullCultureName_ShouldThrowArgumentNullException ( ) {
			var stubStorage = MockRepository.GenerateStub<IStorageProvider> ( );
			var stubCache = MockRepository.GenerateStub<ICacheProvider> ( );

			var sut = new LocalizationProvider ( stubStorage, stubCache );
			using (var readStream = Helper.GetTestModuleStream ( )) {
				Executing.This ( ( ) => sut.SaveModule ( null, "Module1", readStream ) )
					.Should ( )
					.Throw<ArgumentNullException> ( );
			}
		}

		[Test]
		public void SaveModule_WithEmptyModuleName_ShouldThrowArgumentException ( ) {
			var stubStorage = MockRepository.GenerateStub<IStorageProvider> ( );
			var stubCache = MockRepository.GenerateStub<ICacheProvider> ( );

			var sut = new LocalizationProvider ( stubStorage, stubCache );

			using (var readStream = Helper.GetTestModuleStream ( )) {
				Executing.This ( ( ) => sut.SaveModule ( "en-US", "", readStream ) )
					.Should ( )
					.Throw<ArgumentException> ( );
			}
		}

		[Test]
		public void SaveModule_WithNullModuleName_ShouldThrowArgumentNullException ( ) {
			var stubStorage = MockRepository.GenerateStub<IStorageProvider> ( );
			var stubCache = MockRepository.GenerateStub<ICacheProvider> ( );

			var sut = new LocalizationProvider ( stubStorage, stubCache );
			using (var readStream = Helper.GetTestModuleStream ( )) {
				Executing.This ( ( ) => sut.SaveModule ( "en-US", null, readStream ) )
					.Should ( )
					.Throw<ArgumentNullException> ( );
			}
		}

		[Test]
		public void SaveModule_WithNullStream_ShouldThrowArgumentNullException ( ) {
			var stubStorage = MockRepository.GenerateStub<IStorageProvider> ( );
			var stubCache = MockRepository.GenerateStub<ICacheProvider> ( );

			var sut = new LocalizationProvider ( stubStorage, stubCache );
				
			Executing.This ( ( ) => sut.SaveModule ( "en-US", "Module1", null ) )
				.Should ( )
				.Throw<ArgumentNullException> ( );
		}

		[Test]
		public void SaveModule_WithAnExistingModule_ShouldThrowLocalizationModuleExistentException() {
			var stubStorage = MockRepository.GenerateStub<IStorageProvider> ( );
			var stubCache = MockRepository.GenerateStub<ICacheProvider> ( );

			stubCache.Stub ( x => x.Get ( null ) ).IgnoreArguments ( ).Return ( null );
			stubCache.Stub ( x => x.Get ( null, null ) ).IgnoreArguments ( ).Return ( null );

			stubStorage.Stub ( x => x.FileExist ( null ) ).IgnoreArguments ( ).Return ( true );

			var sut = new LocalizationProvider ( stubStorage, stubCache );
			using (var readStream = Helper.GetTestModuleStream ( )) {
				Executing.This ( ( ) => sut.SaveModule ( "en-US", "TestModule", readStream ) )
					.Should ( )
					.Throw<LocalizationModuleExistentException> ( );
			}
		}

		[Test]
		public void SaveModule_WithValidParameter_ShouldCallSaveStreamMethodIntoStorageMock() {
			var mockStorage = MockRepository.GenerateMock<IStorageProvider> ( );
			var stubCache = MockRepository.GenerateStub<ICacheProvider> ( );

			stubCache.Stub ( x => x.Get ( null ) ).IgnoreArguments ( ).Return ( null );
			stubCache.Stub ( x => x.Get ( null, null ) ).IgnoreArguments ( ).Return ( null );

			mockStorage.Stub ( x => x.FileExist ( null ) ).IgnoreArguments ( ).Return ( false );

			var sut = new LocalizationProvider ( mockStorage, stubCache );
			using (var readStream = Helper.GetTestModuleStream ( )) {
				sut.SaveModule ( "en-US" , "NewTestModule" , readStream );

				mockStorage.AssertWasCalled ( x => x.SaveStream ( "Localization/en-US/NewTestModule.po",readStream ) );
			}
		}

		#endregion

		#region SaveOrUpdateModule Method

		[Test]
		public void SaveOrUpdateModule_WithEmptyCultureName_ShouldThrowArgumentException ( ) {
			var stubStorage = MockRepository.GenerateStub<IStorageProvider> ( );
			var stubCache = MockRepository.GenerateStub<ICacheProvider> ( );

			var sut = new LocalizationProvider ( stubStorage, stubCache );

			using (var readStream = Helper.GetTestModuleStream ( )) {
				Executing.This ( ( ) => sut.SaveOrUpdateModule("", "Module1", readStream ) )
					.Should ( )
					.Throw<ArgumentException> ( );
			}
		}

		[Test]
		public void SaveOrUpdateModule_WithNullCultureName_ShouldThrowArgumentNullException ( ) {
			var stubStorage = MockRepository.GenerateStub<IStorageProvider> ( );
			var stubCache = MockRepository.GenerateStub<ICacheProvider> ( );

			var sut = new LocalizationProvider ( stubStorage, stubCache );
			using (var readStream = Helper.GetTestModuleStream ( )) {
				Executing.This ( ( ) => sut.SaveOrUpdateModule ( null, "Module1", readStream ) )
					.Should ( )
					.Throw<ArgumentNullException> ( );
			}
		}

		[Test]
		public void SaveOrUpdateModule_WithEmptyModuleName_ShouldThrowArgumentException ( ) {
			var stubStorage = MockRepository.GenerateStub<IStorageProvider> ( );
			var stubCache = MockRepository.GenerateStub<ICacheProvider> ( );

			var sut = new LocalizationProvider ( stubStorage, stubCache );

			using (var readStream = Helper.GetTestModuleStream ( )) {
				Executing.This ( ( ) => sut.SaveOrUpdateModule ( "en-US", "", readStream ) )
					.Should ( )
					.Throw<ArgumentException> ( );
			}
		}

		[Test]
		public void SaveOrUpdateModule_WithNullModuleName_ShouldThrowArgumentNullException ( ) {
			var stubStorage = MockRepository.GenerateStub<IStorageProvider> ( );
			var stubCache = MockRepository.GenerateStub<ICacheProvider> ( );

			var sut = new LocalizationProvider ( stubStorage, stubCache );
			using (var readStream = Helper.GetTestModuleStream ( )) {
				Executing.This ( ( ) => sut.SaveOrUpdateModule ( "en-US", null, readStream ) )
					.Should ( )
					.Throw<ArgumentNullException> ( );
			}
		}

		[Test]
		public void SaveOrUpdateModule_WithNullStream_ShouldThrowArgumentNullException ( ) {
			var stubStorage = MockRepository.GenerateStub<IStorageProvider> ( );
			var stubCache = MockRepository.GenerateStub<ICacheProvider> ( );

			var sut = new LocalizationProvider ( stubStorage, stubCache );

			Executing.This ( ( ) => sut.SaveOrUpdateModule ( "en-US", "Module1", null ) )
				.Should ( )
				.Throw<ArgumentNullException> ( );
		}

		[Test]
		public void SaveOrUpdateModule_WithValidParameter_ShouldCallSaveStreamMethodIntoStorageMock ( ) {
			var mockStorage = MockRepository.GenerateMock<IStorageProvider> ( );
			var stubCache = MockRepository.GenerateStub<ICacheProvider> ( );

			stubCache.Stub ( x => x.Get ( null ) ).IgnoreArguments ( ).Return ( null );
			stubCache.Stub ( x => x.Get ( null, null ) ).IgnoreArguments ( ).Return ( null );

			mockStorage.Stub ( x => x.FileExist ( null ) ).IgnoreArguments ( ).Return ( false );

			var sut = new LocalizationProvider ( mockStorage, stubCache );
			using (var readStream = Helper.GetTestModuleStream ( )) {
				sut.SaveOrUpdateModule ( "en-US", "NewTestModule", readStream );

				mockStorage.AssertWasCalled ( x => x.SaveStream ( "Localization/en-US/NewTestModule.po", readStream ) );
			}
		}

		#endregion

		#region DeleteModule

		[Test]
		public void DeleteModule_WithEmptyCultureName_ShouldThrowArgumentException ( ) {
			var stubStorage = MockRepository.GenerateStub<IStorageProvider> ( );
			var stubCache = MockRepository.GenerateStub<ICacheProvider> ( );

			var sut = new LocalizationProvider ( stubStorage, stubCache );


			Executing.This ( ( ) => sut.DeleteModule ( "", "Module1" ) )
					.Should ( )
					.Throw<ArgumentException> ( );

		}

		[Test]
		public void DeleteModule_WithNullCultureName_ShouldThrowArgumentNullException ( ) {
			var stubStorage = MockRepository.GenerateStub<IStorageProvider> ( );
			var stubCache = MockRepository.GenerateStub<ICacheProvider> ( );

			var sut = new LocalizationProvider ( stubStorage, stubCache );

			Executing.This ( ( ) => sut.DeleteModule ( null, "Module1" ) )
					.Should ( )
					.Throw<ArgumentNullException> ( );

		}

		[Test]
		public void DeleteModule_WithNotExistingModule_ShouldThrowLocalizationModuleNotFoundException() {
			var mockStorage = MockRepository.GenerateMock<IStorageProvider> ( );
			var stubCache = MockRepository.GenerateStub<ICacheProvider> ( );

			stubCache.Stub ( x => x.Get ( null ) ).IgnoreArguments ( ).Return ( null );
			stubCache.Stub ( x => x.Get ( null, null ) ).IgnoreArguments ( ).Return ( null );

			mockStorage.Stub ( x => x.FileExist ( null ) ).IgnoreArguments ( ).Return ( false );

			var sut = new LocalizationProvider ( mockStorage, stubCache );
			Executing.This ( ( ) => sut.DeleteModule ( "en-US", "Module1" ) )
					.Should ( )
					.Throw<LocalizationModuleNotFoundException> ( );

		}

		#endregion
	}
}