﻿ #region Disclaimer/Info
 
 /////////////////////////////////////////////////////////////////////////////////////////////////
 //
 //   File:		BlogRollRepositoryTest.cs
 //   Website:		http://dexterblogengine.com/
 //   Authors:		http://dexterblogengine.com/About.ashx
 //   Rev:		1
 //   Created:		19/01/2011
 //   Last edit:		19/01/2011
 //   License:		GNU Library General Public License (LGPL)
 //   File:            BlogRollRepositoryTest.cs
 //   For updated news and information please visit http://dexterblogengine.com/
 //   Dexter is hosted to Codeplex at http://dexterblogengine.codeplex.com
 //   For any question contact info@dexterblogengine.com
 //
 ///////////////////////////////////////////////////////////////////////////////////////////////////
 
 #endregion

using System;
using Dexter.Domain.Model;
using Dexter.Logger.Concretes;
using Dexter.Logger.Contracts;
using Dexter.Repository.NHibernate.Repositories;
using Moq;
using NHibernate;
using NUnit.Framework;
using SharpTestsEx;
using ISessionFactory = Dexter.Repository.NHibernate.SessionFactory.ISessionFactory;

namespace Dexter.Repository.NHibernate.Test.Repositories
{
	[TestFixture]
	public class BlogRollRepositoryTest : BaseNHibernateTest
	{
		public BlogRollRepositoryTest ( ) : base ( true ) {}

		[Test]    
		public void SaveToRepository_WithValidData_ShouldPersistTheItemWithoutExceptions ()
		{
			BlogRoll itemToSave = BlogRoll.CreateNewBlogRoll ( "myblog" , new Uri("http://www.tostring.it") );

			using ( ISession session = TestSession )
			using (ITransaction tx = session.BeginTransaction ( ))
			{
				BlogRollRepository classUnderTest = new BlogRollRepository ( Factory , NullLogger.Instance );

				classUnderTest.Save ( itemToSave );

				tx.Commit ();
			}

			using (ISession session = GetNewSession)
			using ( ITransaction tx = session.BeginTransaction () )
			{
				BlogRoll item = session.Get<BlogRoll> ( itemToSave.Id );

				item.Should ().Not.Be.Null ();
			}
		}

		[Test]
		public void SaveToRepository_WithValidData_ShouldCallValidateForSaveMethod()
		{
			var itemToSave = new Mock<BlogRoll> ();
			itemToSave.Setup ( x => x.Name ).Returns ( "tostring.it" );
			itemToSave.Setup(x => x.Link).Returns(new Uri("http://www.tostring.it"));
			itemToSave.Setup(x => x.Id).Returns(0);

			var stubFactory = new Mock<ISessionFactory>();
			stubFactory.Setup(x => x.GetSession()).Returns(new Mock<ISession>().Object);

			BlogRollRepository classUnderTest = new BlogRollRepository(stubFactory.Object, NullLogger.Instance );
			classUnderTest.Save(itemToSave.Object);

			itemToSave.Setup ( its => its.ValidateForSave ()).Verifiable();

			itemToSave.Verify(its => its.ValidateForSave(), Times.Once());
		}

		[Test]
		public void SaveToRepository_WithNullItem_ShouldThrowArgumentNullException()
		{
			Executing.This ( () =>
			                 	{
									using (ISession session = TestSession)
									using (ITransaction tx = session.BeginTransaction ( ))
									{
										BlogRollRepository classUnderTest = new BlogRollRepository(Factory, NullLogger.Instance );

										classUnderTest.Save(null);

										tx.Commit();
									}
			                 	} )
				.Should ()
				.Throw<ArgumentNullException> ()
				.And
				.Exception
				.ParamName
				.Should ()
				.Be
				.EqualTo ( "item" );
		}

		[Test]
		public void DeleteFromRepository_WithNullItem_ShouldThrowArgumentNullException()
		{
			Executing.This(() =>
			{
				using (ISession session = TestSession)
				using (ITransaction tx = session.BeginTransaction ( ))
				{
					BlogRollRepository classUnderTest = new BlogRollRepository(Factory, NullLogger.Instance );

					classUnderTest.Delete ( null );

					tx.Commit();
				}
			})
				.Should()
				.Throw<ArgumentNullException>()
				.And
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("item");
		}

		[Test]
		public void DeleteToRepository_WithValidData_ShouldCallValidateForDelete()
		{
			var itemToSave = new Mock<BlogRoll>();
			itemToSave.Setup(x => x.Name).Returns("tostring.it");
			itemToSave.Setup(x => x.Link).Returns(new Uri("http://www.tostring.it"));
			itemToSave.Setup(x => x.Id).Returns(0);

			var stubFactory = new Mock<ISessionFactory>();
			stubFactory.Setup(x => x.GetSession()).Returns(new Mock<ISession>().Object);

			BlogRollRepository classUnderTest = new BlogRollRepository(stubFactory.Object, NullLogger.Instance );
			classUnderTest.Delete(itemToSave.Object);

			itemToSave.Verify(its => its.ValidateForDelete(), Times.Once());
		}
		
		[Test]
		public void DeleteRepository_WithValidData_ShouldRemoveTheItemWithoutExceptions()
		{
			var itemToSave = this.AddNewBlogRollToRepository ();

			using (ISession session = TestSession)
			using (ITransaction tx = session.BeginTransaction ( ))
			{
				BlogRollRepository classUnderTest = new BlogRollRepository(Factory, NullLogger.Instance );

				classUnderTest.Delete(itemToSave);

				tx.Commit();
			}

			using (ISession session = GetNewSession)
			using (ITransaction tx = session.BeginTransaction())
			{
				BlogRoll item = session.Get<BlogRoll>(itemToSave.Id);

				item.Should().Be.Null();
			}
		}

		[Test]
		public void GetListPaged_WithNegativePageIndex_ShouldThrowNewArgumentOutOfRangeException()
		{
			this.AddNewBlogRollsToRepository(5);

			Executing.This ( () =>
			                 	{
									using (ISession session = TestSession)
									using (ITransaction tx = session.BeginTransaction ( ))
									{
										BlogRollRepository classUnderTest = new BlogRollRepository(Factory, NullLogger.Instance );

										classUnderTest.GetListPaged ( -5 , 10 );
									}
			                 	} )
				.Should ()
				.Throw<ArgumentOutOfRangeException> ()
				.Exception
				.ParamName
				.Should ()
				.Be
				.EqualTo ( "pageIndex" );
		}

		[Test]
		public void GetListPaged_WithZeroPageIndex_ShouldExecutedWithoutExceptions()
		{
			this.AddNewBlogRollsToRepository(5);

			Executing.This ( () =>
			                 	{
			                 		using ( ISession session = TestSession )
									using (ITransaction tx = session.BeginTransaction ( ))
			                 		{
										BlogRollRepository classUnderTest = new BlogRollRepository ( Factory, NullLogger.Instance );

			                 			classUnderTest.GetListPaged ( 0 , 10 );
			                 		}
			                 	} )
				.Should ()
				.NotThrow ();
		}

		[Test]
		public void GetListPaged_WithNegativePageSize_ShouldThrowNewArgumentOutOfRangeException()
		{
			this.AddNewBlogRollsToRepository(5);

			Executing.This(() =>
			{
				using (ISession session = TestSession)
				using (ITransaction tx = session.BeginTransaction())
				{
					BlogRollRepository classUnderTest = new BlogRollRepository(Factory, NullLogger.Instance );

					classUnderTest.GetListPaged(1, -10);
				}
			})
				.Should()
				.Throw<ArgumentOutOfRangeException>()
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("pageSize");
		}

		[Test]
		public void GetList_PagedWithZeroPageSize_ShouldExecutedWithoutExceptions()
		{
			this.AddNewBlogRollsToRepository(5);
			Executing.This(() =>
			{
				using (ISession session = TestSession)
				using (ITransaction tx =  session.BeginTransaction())
				{
					BlogRollRepository classUnderTest = new BlogRollRepository(Factory, NullLogger.Instance );

					classUnderTest.GetListPaged(1, 10);
				}
			})
				.Should()
				.NotThrow();
		}
	}
}
