﻿#region Disclaimer/Info

/////////////////////////////////////////////////////////////////////////////////////////////////
//
//   File:		CategoryRepositoryTest.cs
//   Website:		http://dexterblogengine.com/
//   Authors:		http://dexterblogengine.com/About.ashx
//   Rev:		1
//   Created:		19/01/2011
//   Last edit:		19/01/2011
//   License:		GNU Library General Public License (LGPL)
//   File:            CategoryRepositoryTest.cs
//   For updated news and information please visit http://dexterblogengine.com/
//   Dexter is hosted to Codeplex at http://dexterblogengine.codeplex.com
//   For any question contact info@dexterblogengine.com
//
///////////////////////////////////////////////////////////////////////////////////////////////////

#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using Dexter.Domain.Model;
using Dexter.Logger.Concretes;
using Dexter.Logger.Contracts;
using Dexter.Repository.NHibernate.Repositories;
using Moq;
using NHibernate;
using NUnit.Framework;
using SharpTestsEx;
using ISessionFactory = Dexter.Repository.NHibernate.SessionFactory.ISessionFactory;

namespace Dexter.Repository.NHibernate.Test.Repositories
{
	[TestFixture]
	public class CategoryRepositoryTest : BaseNHibernateTest
	{
		public CategoryRepositoryTest() : base(true) { }

		[Test]
		public void DeleteCategory_ShouldCallValidateOnDeleteMethod()
		{
			var itemToDelete = new Mock<Category>();
			itemToDelete.Setup(x => x.Name).Returns("my category");
			itemToDelete.Setup(x => x.Id).Returns(1);

			var stubFactory = new Mock<ISessionFactory>();
			stubFactory.Setup(x => x.GetSession()).Returns(new Mock<ISession>().Object);

			var classUnderTest = new CategoryRepository(stubFactory.Object, NullLogger.Instance);
			try
			{
				classUnderTest.DeleteCategory(itemToDelete.Object, this.GetNewCategory(null));
			}
			catch (Exception e)
			{
				Console.WriteLine(e);
			}

			itemToDelete.Verify(its => its.ValidateForDelete(),Times.Once());
		}

		[Test]
		public void DeleteCategory_ShouldMoveAllPostsIntoDefaultCategory()
		{
			Category defaultCategory = this.AddDefaultCategoryToRepository();
			Category postCategory = this.AddNewCategoryToRepository();

			Post actual = this.AddNewPostToRepository(new[] { postCategory });

			using (ISession session = TestSession)
			{
				using (ITransaction tx = session.BeginTransaction())
				{
					var classUnderTest = new CategoryRepository(Factory, NullLogger.Instance);
					var category2Delete = TestSession.Get<Category>(postCategory.Id);

					classUnderTest.DeleteCategory(category2Delete, null);

					tx.Commit();
				}
			}

			using (ISession session = GetNewSession)
			{
				using (ITransaction tx = session.BeginTransaction())
				{
					var expected = session.Get<Post>(actual.Id);
					expected.Categories.Count.Should().Be.EqualTo(1);

					expected.Categories[0].Id.Should().Be.EqualTo(defaultCategory.Id);
				}
			}
		}

		[Test]
		public void DeleteCategory_WithNullObject_ShouldThrowArgumentNullException()
		{
			Executing.This(() =>
			{
				using (ISession session = TestSession)
				{
					using (ITransaction tx = session.BeginTransaction())
					{
						var classUnderTest = new CategoryRepository(Factory, NullLogger.Instance);

						classUnderTest.DeleteCategory(null, null);
					}
				}
			})
				.Should()
				.Throw<ArgumentNullException>()
				.And
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("category");
		}

		[Test]
		public void GetAllCategories_ShoudlReturnTheCorrectList()
		{
			Category c1 = this.GetNewCategory(null, true);
			Category c2 = this.GetNewCategory(c1, false);
			Category c3 = this.GetNewCategory(c1, false);
			Category c4 = this.GetNewCategory(null, false);

			this.Persist(c1, c2, c3, c4);

			IEnumerable<Category> categories;

			using (ISession session = TestSession)
			{
				using (ITransaction tx = session.BeginTransaction())
				{
					var classUnderTest = new CategoryRepository(Factory, NullLogger.Instance);

					categories = classUnderTest.GetAllCategories();

					categories.Should().Not.Be.Null();
					categories.Count().Should().Be.EqualTo(4);
				}
			}
		}

		[Test]
		public void GetCategoryByKey_ShouldThrowNewArgumentOutOfRangeException_WithInvalidId()
		{
			Executing.This(() =>
			{
				using (ISession session = TestSession)
				{
					using (ITransaction tx = session.BeginTransaction())
					{
						var classUnderTest = new CategoryRepository(Factory, NullLogger.Instance);

						classUnderTest.GetCategoryByKey(0);
					}
				}
			})
				.Should()
				.Throw<ArgumentOutOfRangeException>()
				.And
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("key");
		}

		[Test]
		public void GetCategoryByKey_WithChildsShouldNotSufferFromNPlusOneProblem()
		{
			Category c1 = this.GetNewCategory();
			Category c2 = this.GetNewCategory(c1);
			Category c3 = this.GetNewCategory(c1);

			this.Persist(c1, c2, c3);

			Category loaded;
			using (ISession session = GetNewSession)
			{
				using (ITransaction tx = session.BeginTransaction())
				{
					var classUnderTest = new CategoryRepository(Factory, NullLogger.Instance);

					loaded = classUnderTest.GetCategoryByKeyWithChilds(c1.Id);
				}
			}

			loaded.Should().Not.Be.Null();
			loaded.Categories.Should().Not.Be.Null();
			loaded.Categories.Count.Should().Be.EqualTo(2);

			Executing.This(() =>
			{
				//if the fetch was eager, now that the session is close we should
				//not find any problem in iterating categories)))))
				foreach (Category category in loaded.Categories)
				{
					Console.WriteLine(category.Name);
				}
			})
				.Should()
				.NotThrow();
		}

		[Test]
		public void GetCategoryByKey_WithChildsShouldThrowNewArgumentOutOfRangeException_WithInvalidId()
		{
			Executing.This(() =>
			{
				using (ISession session = TestSession)
				{
					using (ITransaction tx = TestSession.BeginTransaction())
					{
						var classUnderTest = new CategoryRepository(Factory, NullLogger.Instance);

						classUnderTest.GetCategoryByKeyWithChilds(0);
					}
				}
			})
				.Should()
				.Throw<ArgumentOutOfRangeException>()
				.And
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("key");
		}

		[Test]
		public void GetDefaultCategory_ShouldReturnTheDefaultCategoryFromRepository()
		{
			Category saved = this.AddDefaultCategoryToRepository();
			Category retrieved;

			using (ISession session = TestSession)
			{
				using (ITransaction tx = TestSession.BeginTransaction())
				{
					var classUnderTest = new CategoryRepository(Factory, NullLogger.Instance);

					retrieved = classUnderTest.GetDefaultCategory();
				}
			}

			retrieved
				.Should()
				.Be
				.EqualTo(saved);
		}

		[Test]
		public void GetDefaultCategory_WithChildsShouldNotSufferFromNPlusOneProblem()
		{
			Category c1 = this.GetNewCategory(null, true);
			Category c2 = this.GetNewCategory(c1);
			Category c3 = this.GetNewCategory(c1);

			this.Persist(c1, c2, c3);

			Category loaded;
			using (ISession session = GetNewSession)
			{
				using (ITransaction tx = session.BeginTransaction())
				{
					var classUnderTest = new CategoryRepository(Factory, NullLogger.Instance);

					loaded = classUnderTest.GetDefaultCategoryWithChilds();
				}
			}

			loaded.Should().Not.Be.Null();
			loaded.Categories.Should().Not.Be.Null();
			loaded.Categories.Count.Should().Be.EqualTo(2);

			Executing.This(() =>
			{
				//if the fetch was eager, now that the session is close we should
				//not find any problem in iterating categories)))))
				foreach (Category category in loaded.Categories)
				{
					Console.WriteLine(category.Name);
				}
			})
				.Should()
				.NotThrow();
		}

		[Test]
		public void Save_ShouldPersistTheItemWithoutExceptions()
		{
			Category itemToSave = Category.CreateNewCategory("myblog", null);

			using (ISession session = TestSession)
			{
				using (ITransaction tx = TestSession.BeginTransaction())
				{
					var classUnderTest = new CategoryRepository(Factory, NullLogger.Instance);

					classUnderTest.Save(itemToSave);

					tx.Commit();
				}
			}

			using (ISession session = GetNewSession)
			{
				using (ITransaction tx = session.BeginTransaction())
				{
					var item = session.Get<Category>(itemToSave.Id);

					item.Should().Not.Be.Null();
					item.Id.Should().Be.EqualTo(item.Id);
					item.Name.Should().Be.EqualTo(item.Name);
				}
			}
		}

		[Test]
		public void Save_WithNewDefaultCategory_ShouldUpdateOldDefaultCategory()
		{
			Category defaultCategory = this.AddDefaultCategoryToRepository();
			Category newDefaultCategory = this.GetNewCategory(null, true);

			using (ISession session = TestSession)
			{
				using (ITransaction tx = TestSession.BeginTransaction())
				{
					var classUnderTest = new CategoryRepository(Factory, NullLogger.Instance);

					classUnderTest.Save(newDefaultCategory);

					tx.Commit();
				}
			}

			using (ISession session = GetNewSession)
			{
				using (ITransaction tx = session.BeginTransaction())
				{
					var oldDefaultCategory = session.Get<Category>(defaultCategory.Id);

					oldDefaultCategory.IsDefaultCategory
						.Should()
						.Be
						.False();
				}
			}
		}

		[Test]
		public void Save_WithNotNullItem_ShouldCallValidateForSave()
		{
			var itemToSave = new Mock<Category>();
			itemToSave.Setup(x => x.Name).Returns("My Category");
			itemToSave.Setup(x => x.Id).Returns(0);

			var stubFactory = new Mock<ISessionFactory>();
			stubFactory.Setup(x => x.GetSession()).Returns(new Mock<ISession>().Object);

			var classUnderTest = new CategoryRepository(stubFactory.Object, NullLogger.Instance);
			classUnderTest.Save(itemToSave.Object);

			itemToSave.Verify(its => its.ValidateForSave(), Times.Once());
		}

		[Test]
		public void Save_WithNullItem_ShouldThrowArgumentNullException()
		{
			Executing.This(() =>
			{
				using (ISession session = GetNewSession)
				{
					using (ITransaction tx = session.BeginTransaction())
					{
						var classUnderTest = new CategoryRepository(Factory, NullLogger.Instance);

						classUnderTest.Save(null);
					}
				}
			})
				.Should()
				.Throw<ArgumentNullException>()
				.And
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("category");
		}

		[Test]
		public void Update_ShouldPersistTheItemWithoutExceptions()
		{
			Category itemToSave = this.GetNewCategory();

			this.Persist(itemToSave);

			using (ISession session = TestSession)
			{
				using (ITransaction tx = TestSession.BeginTransaction())
				{
					var classUnderTest = new CategoryRepository(Factory, NullLogger.Instance);

					classUnderTest.Update(itemToSave);

					tx.Commit();
				}
			}

			using (ISession session = GetNewSession)
			{
				using (ITransaction tx = session.BeginTransaction())
				{
					var item = session.Get<Category>(itemToSave.Id);

					item.Should().Not.Be.Null();
					item.Id.Should().Be.EqualTo(item.Id);
					item.Name.Should().Be.EqualTo(item.Name);
				}
			}
		}

		[Test]
		public void Update_WithNotNullItem_ShouldCallValidateForUpdate()
		{
			var itemToSave = new Mock<Category>();
			itemToSave.Setup(x => x.Name).Returns("My Category");
			itemToSave.Setup(x => x.Id).Returns(0);

			var stubFactory = new Mock<ISessionFactory>();
			stubFactory.Setup(x => x.GetSession()).Returns(new Mock<ISession>().Object);

			var classUnderTest = new CategoryRepository(stubFactory.Object, NullLogger.Instance);
			classUnderTest.Update(itemToSave.Object);

			itemToSave.Verify(its => its.ValidateForUpdate(), Times.Once());
		}

		[Test]
		public void Update_WithNullItem_ShouldThrowArgumentNullException()
		{
			Executing.This(() =>
			{
				using (ISession session = GetNewSession)
				{
					using (ITransaction tx = session.BeginTransaction())
					{
						var classUnderTest = new CategoryRepository(Factory, NullLogger.Instance);

						classUnderTest.Update(null);
					}
				}
			})
				.Should()
				.Throw<ArgumentNullException>()
				.And
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("category");
		}
	}
}
