﻿#region Disclaimer/Info

/////////////////////////////////////////////////////////////////////////////////////////////////
//
//   File:		EmailRepositoryTest.cs
//   Website:		http://dexterblogengine.com/
//   Authors:		http://dexterblogengine.com/About.ashx
//   Rev:		1
//   Created:		19/01/2011
//   Last edit:		19/01/2011
//   License:		GNU Library General Public License (LGPL)
//   File:            EmailRepositoryTest.cs
//   For updated news and information please visit http://dexterblogengine.com/
//   Dexter is hosted to Codeplex at http://dexterblogengine.codeplex.com
//   For any question contact info@dexterblogengine.com
//
///////////////////////////////////////////////////////////////////////////////////////////////////

#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using Dexter.Domain.Model;
using Dexter.Logger.Concretes;
using Dexter.Logger.Contracts;
using Dexter.Repository.NHibernate.Repositories;
using Moq;
using NHibernate;
using NUnit.Framework;
using SharpTestsEx;
using ISessionFactory = Dexter.Repository.NHibernate.SessionFactory.ISessionFactory;

namespace Dexter.Repository.NHibernate.Test.Repositories
{
	[TestFixture]
	public class EmailRepositoryTest : BaseNHibernateTest
	{
		public EmailRepositoryTest() : base(true) { }

		[Test]
		public void Queue_WithNullObjec_ShouldThrownArgumentNullException()
		{
			Executing.This(() =>
								{
									using (ISession session = TestSession)
									using (ITransaction tx = session.BeginTransaction())
									{
										EmailRepository classUnderTest = new EmailRepository(Factory, NullLogger.Instance);

										classUnderTest.Queue(null);

										tx.Commit();
									}
								})
				.Should()
				.Throw<ArgumentNullException>()
				.And
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("item");
		}

		[Test]
		public void Queue_WithNotNullEntity_ShouldCallValidateMethod()
		{
			var itemToSave = new Mock<EmailMessage>();
			itemToSave.Setup(x => x.Id).Returns(0);

			var stubFactory = new Mock<ISessionFactory>();
			stubFactory.Setup(x => x.GetSession()).Returns(new Mock<ISession>().Object);

			EmailRepository classUnderTest = new EmailRepository(stubFactory.Object, NullLogger.Instance);
			try
			{
				classUnderTest.Queue(itemToSave.Object);
			}
			catch
			{
				//catch all exception because are not important for the test
				//what happening after the validation method call
			}

			itemToSave.Verify(its => its.ValidateForQueue(), Times.Once());
		}

		[Test]
		public void Queue_WithValidaData_ShouldWorksCorrectly()
		{
			var email = this.GetNewEmail();

			Executing.This(() =>
			{
				using (ISession session = TestSession)
				using (ITransaction tx = session.BeginTransaction())
				{
					EmailRepository classUnderTest = new EmailRepository(Factory, NullLogger.Instance);

					classUnderTest.Queue(email);

					tx.Commit();
				}
			})
			.Should()
			.NotThrow();

			email.Id.Should().Be.GreaterThan(0);
		}

		[Test]
		public void Dequeue_WithNullObjec_ShouldThrownArgumentNullException()
		{
			Executing.This(() =>
			{
				using (ISession session = TestSession)
				using (ITransaction tx = session.BeginTransaction())
				{
					EmailRepository classUnderTest = new EmailRepository(Factory, NullLogger.Instance);

					classUnderTest.Dequeue(null);

					tx.Commit();
				}
			})
				.Should()
				.Throw<ArgumentNullException>()
				.And
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("item");
		}

		[Test]
		public void Dequeue_WithNotNullEntity_ShouldCallValidateMethod()
		{
			var itemToSave = new Mock<EmailMessage>();
			itemToSave.Setup(x => x.Id).Returns(0);

			var stubFactory = new Mock<ISessionFactory>();
			stubFactory.Setup(x => x.GetSession()).Returns(new Mock<ISession>().Object);

			EmailRepository classUnderTest = new EmailRepository(stubFactory.Object, NullLogger.Instance);
			try
			{
				classUnderTest.Dequeue(itemToSave.Object);
			}
			catch
			{
				//catch all exception because are not important for the test
				//what happening after the validation method call
			}

			itemToSave.Verify(its => its.ValidateForDequeue(), Times.Once());
		}

		[Test]
		public void Dequeue_WithValidaData_ShouldWorksCorrectly()
		{
			var email = this.AddNewEmailMessagesToRepository(5);

			Executing.This(() =>
			{
				using (ISession session = TestSession)
				using (ITransaction tx = session.BeginTransaction())
				{
					EmailRepository classUnderTest = new EmailRepository(Factory, NullLogger.Instance);

					classUnderTest.Dequeue(email[0]);

					tx.Commit();
				}
			})
			.Should()
			.NotThrow();

			using (ISession session = GetNewSession)
			using (ITransaction tx = session.BeginTransaction())
			{
				var repository = session.CreateQuery("from EmailMessage")
					.List<EmailMessage>();

				repository.Count.Should().Be.EqualTo(4);
				repository.Where(x => x.Id == email[0].Id).Count().Should().Be.EqualTo(0);
			}
		}

		[Test]
		public void Peek_WithNegativeSize_ShouldThrowArgumentOutOfRangeException()
		{
			Executing.This(() =>
			{
				using (ISession session = TestSession)
				using (ITransaction tx = session.BeginTransaction())
				{
					EmailRepository classUnderTest = new EmailRepository(Factory, NullLogger.Instance);

					classUnderTest.Peek(-5);

					tx.Commit();
				}
			})
				.Should()
				.Throw<ArgumentOutOfRangeException>()
				.And
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("size");
		}

		[Test]
		public void Peek_WithValidaData_ShouldWorksCorrectly()
		{
			this.AddNewEmailMessagesToRepository(5);

			Executing.This(() =>
			{
				using (ISession session = TestSession)
				using (ITransaction tx = session.BeginTransaction())
				{
					EmailRepository classUnderTest = new EmailRepository(Factory, NullLogger.Instance);

					IEnumerable<EmailMessage> values = classUnderTest.Peek(3);

					var a = values.ToList();
					a.Count.Should().Be.EqualTo(3);

					a[0].Should().Not.Be.Null();
					a[0].Subject.Should().Not.Be.Null();
					a[1].Should().Not.Be.Null();
					a[1].Subject.Should().Not.Be.Null();
					a[2].Subject.Should().Not.Be.Null();
					a[2].Subject.Should().Not.Be.Null();
				}
			})
				.Should()
				.NotThrow();
		}
	}
}
