﻿#region Disclaimer/Info

/////////////////////////////////////////////////////////////////////////////////////////////////
//
//   File:		PostRepositoryTest.cs
//   Website:		http://dexterblogengine.com/
//   Authors:		http://dexterblogengine.com/About.ashx
//   Rev:		1
//   Created:		19/01/2011
//   Last edit:		19/01/2011
//   License:		GNU Library General Public License (LGPL)
//   File:            PostRepositoryTest.cs
//   For updated news and information please visit http://dexterblogengine.com/
//   Dexter is hosted to Codeplex at http://dexterblogengine.codeplex.com
//   For any question contact info@dexterblogengine.com
//
///////////////////////////////////////////////////////////////////////////////////////////////////

#endregion

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using Dexter.Domain.Model;
using Dexter.Logger.Concretes;
using Dexter.Logger.Contracts;
using Dexter.Repository.NHibernate.Repositories;
using NHibernate;
using NUnit.Framework;
using SharpTestsEx;

namespace Dexter.Repository.NHibernate.Test.Repositories
{
	[TestFixture]
	public class PostRepositoryTest : BaseNHibernateTest
	{
		public PostRepositoryTest() : base(true) { }

		#region Method GetPublished

		[Test]
		public void GetPublished_WithNegativePageIndex_ShouldThrowArgumentOutOfRangeException()
		{
			Executing.This(() =>
						   {
							   using (ISession session = TestSession)
							   using (ITransaction tx = session.BeginTransaction())
							   {
								   var classUnderTest = new PostRepository(Factory, NullLogger.Instance);

								   classUnderTest.GetPublished(-5, 10);
							   }
						   })
				.Should()
				.Throw<ArgumentOutOfRangeException>()
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("pageIndex");
		}

		[Test]
		public void GetPublished_WithNegativePageSize_ShouldThrowNewArgumentOutOfRangeException()
		{
			Executing.This(() =>
						   {
							   using (ISession session = TestSession)
							   using (ITransaction tx = session.BeginTransaction())
							   {
								   var classUnderTest = new PostRepository(Factory, NullLogger.Instance);

								   classUnderTest.GetPublished(1, -10);
							   }
						   })
				.Should()
				.Throw<ArgumentOutOfRangeException>()
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("pageSize");
		}

		[Test]
		public void GetPublished_WithValidParametersAndEmptyRepository_ShouldNotReturnNull()
		{
			using (ISession session = TestSession)
			using (ITransaction tx = session.BeginTransaction())
			{
				var sut = new PostRepository(Factory, NullLogger.Instance);

				var result = sut.GetPublished(0, 10);

				result.Should().Not.Be.Null();
				result.TotalCount.Should().Be.EqualTo(0);
			}
		}

		[Test]
		public void GetPublished_WithValidParameters_ShouldReturnValidaData()
		{
			var category = this.AddDefaultCategoryToRepository();

			//Adding past posts
			for (int i = 0; i < 3; i++)
			{
				this.AddNewPostToRepository("Title" + Guid.NewGuid(), Guid.NewGuid().ToString(), "abstract", "body", true, DateTime.Now.AddDays(-1 * i), new[] { category });
			}

			//Adding futures posts
			this.AddNewPostToRepository("Title" + Guid.NewGuid(), Guid.NewGuid().ToString(), "abstract", "body", true, DateTime.Now.AddDays(5), new[] { category });

			//Adding unpublished posts
			var unpublishedPosts = this.AddNewPostsToRepository(false, new[] { category }, 3);

			using (ISession session = TestSession)
			using (ITransaction tx = session.BeginTransaction())
			{
				var sut = new PostRepository(Factory, NullLogger.Instance);

				var pagedResult = sut.GetPublished(0, 2);

				pagedResult.Result.Should().Not.Be.Null();
				pagedResult.Result.Count().Should().Be.EqualTo(2);
				pagedResult.TotalCount.Should().Be.EqualTo(3);
			}
		}

		[Test]
		public void GetPublished_WithFutures_ShouldReturnValidaData()
		{
			var category = this.AddDefaultCategoryToRepository();

			//Adding past posts
			for (int i = 0; i < 3; i++)
			{
				this.AddNewPostToRepository("Title" + Guid.NewGuid(), Guid.NewGuid().ToString(), "abstract", "body", true, DateTime.Now.AddDays(-1 * i), new[] { category });
			}

			//Adding futures posts
			this.AddNewPostToRepository("Title" + Guid.NewGuid(), Guid.NewGuid().ToString(), "abstract", "body", true, DateTime.Now.AddDays(5), new[] { category });

			//Adding unpublished posts
			var unpublishedPosts = this.AddNewPostsToRepository(false, new[] { category }, 3);

			using (ISession session = TestSession)
			using (ITransaction tx = session.BeginTransaction())
			{
				var sut = new PostRepository(Factory, NullLogger.Instance);

				var pagedResult = sut.GetPublished(0, 2, true);

				pagedResult.Result.Should().Not.Be.Null();
				pagedResult.Result.Count().Should().Be.EqualTo(2);
				pagedResult.TotalCount.Should().Be.EqualTo(4);
			}
		}

		#endregion

		#region Method GetUnapproved

		[Test]
		public void GetUnpublished_WithNegativePageIndex_ShouldThrowArgumentOutOfRangeException()
		{
			Executing.This(() =>
			{
				using (ISession session = TestSession)
				using (ITransaction tx = session.BeginTransaction())
				{
					var classUnderTest = new PostRepository(Factory, NullLogger.Instance);

					classUnderTest.GetUnpublished(-5, 10);
				}
			})
				.Should()
				.Throw<ArgumentOutOfRangeException>()
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("pageIndex");
		}

		[Test]
		public void GetUnpublished_WithNegativePageSize_ShouldThrowNewArgumentOutOfRangeException()
		{
			Executing.This(() =>
			{
				using (ISession session = TestSession)
				using (ITransaction tx = session.BeginTransaction())
				{
					var classUnderTest = new PostRepository(Factory, NullLogger.Instance);

					classUnderTest.GetUnpublished(1, -10);
				}
			})
				.Should()
				.Throw<ArgumentOutOfRangeException>()
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("pageSize");
		}

		[Test]
		public void GetUnpublished_WithValidParametersAndEmptyRepository_ShouldNotReturnNull()
		{
			using (ISession session = TestSession)
			using (ITransaction tx = session.BeginTransaction())
			{
				var sut = new PostRepository(Factory, NullLogger.Instance);

				var result = sut.GetUnpublished(0, 10);

				result.Should().Not.Be.Null();
				result.TotalCount.Should().Be.EqualTo(0);
			}
		}

		[Test]
		public void GetUnpublished_WithValidParameters_ShouldReturnValidaData()
		{
			var category = this.AddDefaultCategoryToRepository();

			//Adding past published posts
			for (int i = 0; i < 3; i++)
			{
				this.AddNewPostToRepository("Title" + Guid.NewGuid(), Guid.NewGuid().ToString(), "abstract", "body", true, DateTime.Now.AddDays(-1 * i), new[] { category });
			}

			//Adding published futures posts
			this.AddNewPostToRepository("Title" + Guid.NewGuid(), Guid.NewGuid().ToString(), "abstract", "body", true, DateTime.Now.AddDays(5), new[] { category });

			//Adding unpublished futures posts
			this.AddNewPostToRepository("Title" + Guid.NewGuid(), Guid.NewGuid().ToString(), "abstract", "body", false, DateTime.Now.AddDays(5), new[] { category });

			//Adding unpublished posts
			var unpublishedPosts = this.AddNewPostsToRepository(false, new[] { category }, 3);

			using (ISession session = TestSession)
			using (ITransaction tx = session.BeginTransaction())
			{
				var sut = new PostRepository(Factory, NullLogger.Instance);

				var pagedResult = sut.GetUnpublished(0, 2);

				pagedResult.Result.Should().Not.Be.Null();
				pagedResult.Result.Count().Should().Be.EqualTo(2);
				pagedResult.TotalCount.Should().Be.EqualTo(3);
			}
		}

		[Test]
		public void GetUnpublished_WithFutures_ShouldReturnValidaData()
		{
			var category = this.AddDefaultCategoryToRepository();

			//Adding past published posts
			for (int i = 0; i < 3; i++)
			{
				this.AddNewPostToRepository("Title" + Guid.NewGuid(), Guid.NewGuid().ToString(), "abstract", "body", true, DateTime.Now.AddDays(-1 * i), new[] { category });
			}

			//Adding published futures posts
			this.AddNewPostToRepository("Title" + Guid.NewGuid(), Guid.NewGuid().ToString(), "abstract", "body", true, DateTime.Now.AddDays(5), new[] { category });

			//Adding unpublished futures posts
			this.AddNewPostToRepository("Title" + Guid.NewGuid(), Guid.NewGuid().ToString(), "abstract", "body", false, DateTime.Now.AddDays(5), new[] { category });

			//Adding unpublished posts
			var unpublishedPosts = this.AddNewPostsToRepository(false, new[] { category }, 3);

			using (ISession session = TestSession)
			using (ITransaction tx = session.BeginTransaction())
			{
				var sut = new PostRepository(Factory, NullLogger.Instance);

				var pagedResult = sut.GetUnpublished(0, 2, true);

				pagedResult.Result.Should().Not.Be.Null();
				pagedResult.Result.Count().Should().Be.EqualTo(2);
				pagedResult.TotalCount.Should().Be.EqualTo(4);
			}
		}

		#endregion

		#region Method GetCompleteList

		[Test]
		public void GetCompleteList_WithNegativePageIndex_ShouldThrowArgumentOutOfRangeException()
		{
			Executing.This(() =>
			{
				using (ISession session = TestSession)
				using (ITransaction tx = session.BeginTransaction())
				{
					var classUnderTest = new PostRepository(Factory, NullLogger.Instance);

					classUnderTest.GetCompleteList(-5, 10);
				}
			})
				.Should()
				.Throw<ArgumentOutOfRangeException>()
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("pageIndex");
		}

		[Test]
		public void GetCompleteList_WithNegativePageSize_ShouldThrowNewArgumentOutOfRangeException()
		{
			Executing.This(() =>
			{
				using (ISession session = TestSession)
				using (ITransaction tx = session.BeginTransaction())
				{
					var classUnderTest = new PostRepository(Factory, NullLogger.Instance);

					classUnderTest.GetCompleteList(1, -10);
				}
			})
				.Should()
				.Throw<ArgumentOutOfRangeException>()
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("pageSize");
		}

		[Test]
		public void GetCompleteList_WithValidParametersAndEmptyRepository_ShouldNotReturnNull()
		{
			using (ISession session = TestSession)
			using (ITransaction tx = session.BeginTransaction())
			{
				var sut = new PostRepository(Factory, NullLogger.Instance);

				var result = sut.GetCompleteList(0, 10);

				result.Should().Not.Be.Null();
				result.TotalCount.Should().Be.EqualTo(0);
			}
		}

		[Test]
		public void GetCompleteList_WithValidParameters_ShouldReturnValidaData()
		{
			var category = this.AddDefaultCategoryToRepository();
			var publishedPosts = this.AddNewPostsToRepository(true, new[] { category }, 5);
			var unpublishedPosts = this.AddNewPostsToRepository(false, new[] { category }, 3);

			using (ISession session = TestSession)
			using (ITransaction tx = session.BeginTransaction())
			{
				var sut = new PostRepository(Factory, NullLogger.Instance);

				var pagedResult = sut.GetCompleteList(0, 2);

				pagedResult.Result.Should().Not.Be.Null();
				pagedResult.Result.Count().Should().Be.EqualTo(2);
				pagedResult.TotalCount.Should().Be.EqualTo(8);
			}
		}

		#endregion

		#region Method GetRecent

		[Test]
		public void GetRecent_WithNegativenumberOfItems_ShouldThrowArgumentOutOfRangeException()
		{
			Executing.This(() =>
			{
				using (ISession session = TestSession)
				using (ITransaction tx = session.BeginTransaction())
				{
					var classUnderTest = new PostRepository(Factory, NullLogger.Instance);

					classUnderTest.GetRecent(-5, true);
				}
			})
				.Should()
				.Throw<ArgumentOutOfRangeException>()
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("numberOfItems");
		}

		[Test]
		public void GetRecent_WithValidParameters_ShouldReturnValidaData()
		{
			var category = this.AddDefaultCategoryToRepository();
			var repositoryPosts = new Collection<Post>();

			for (int i = 0; i < 5; i++)
			{
				var post = this.AddNewPostToRepository("Title" + Guid.NewGuid(), Guid.NewGuid().ToString(), "abstract", "body", true, DateTime.Now.AddDays(-1 * i), new[] { category });
				repositoryPosts.Add(post);
			}

			using (ISession session = TestSession)
			using (ITransaction tx = session.BeginTransaction())
			{
				var sut = new PostRepository(Factory, NullLogger.Instance);

				var dataResult = sut.GetRecent(2, false);

				dataResult.Should().Not.Be.Null();
				dataResult.Count().Should().Be.EqualTo(2);
				dataResult.ToList()[0].Id.Should().Be.EqualTo(repositoryPosts[0].Id);
				dataResult.ToList()[1].Id.Should().Be.EqualTo(repositoryPosts[1].Id);
			}
		}

		#endregion

		#region Method GetByMonth

		[Test]
		public void GetByMonth_WithNegativePageIndex_ShouldThrowArgumentOutOfRangeException()
		{
			Executing.This(() =>
			{
				using (ISession session = TestSession)
				using (ITransaction tx = session.BeginTransaction())
				{
					var classUnderTest = new PostRepository(Factory, NullLogger.Instance);

					classUnderTest.GetByMonth(1, 2010, -5, 10);
				}
			})
				.Should()
				.Throw<ArgumentOutOfRangeException>()
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("pageIndex");
		}

		[Test]
		public void GetByMonth_WithNegativePageSize_ShouldThrowNewArgumentOutOfRangeException()
		{
			Executing.This(() =>
			{
				using (ISession session = TestSession)
				using (ITransaction tx = session.BeginTransaction())
				{
					var classUnderTest = new PostRepository(Factory, NullLogger.Instance);

					classUnderTest.GetByMonth(1, 2010, 1, -10);
				}
			})
				.Should()
				.Throw<ArgumentOutOfRangeException>()
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("pageSize");
		}

		[Test]
		public void GetByMonth_WithNegativeMonthValue_ShouldThrowNewArgumentOutOfRangeException()
		{
			Executing.This(() =>
			{
				using (ISession session = TestSession)
				using (ITransaction tx = session.BeginTransaction())
				{
					var classUnderTest = new PostRepository(Factory, NullLogger.Instance);

					classUnderTest.GetByMonth(-5, 2010, 1, 10);
				}
			})
				.Should()
				.Throw<ArgumentOutOfRangeException>()
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("month");
		}

		[Test]
		public void GetByMonth_WithZeroMonthValue_ShouldThrowNewArgumentOutOfRangeException()
		{
			Executing.This(() =>
			{
				using (ISession session = TestSession)
				using (ITransaction tx = session.BeginTransaction())
				{
					var classUnderTest = new PostRepository(Factory, NullLogger.Instance);

					classUnderTest.GetByMonth(0, 2010, 1, 10);
				}
			})
				.Should()
				.Throw<ArgumentOutOfRangeException>()
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("month");
		}

		[Test]
		public void GetByMonth_WithMonthValueGreaterThen12_ShouldThrowNewArgumentOutOfRangeException()
		{
			Executing.This(() =>
			{
				using (ISession session = TestSession)
				using (ITransaction tx = session.BeginTransaction())
				{
					var classUnderTest = new PostRepository(Factory, NullLogger.Instance);

					classUnderTest.GetByMonth(13, 2010, 1, 10);
				}
			})
				.Should()
				.Throw<ArgumentOutOfRangeException>()
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("month");
		}

		[Test]
		public void GetByMonth_WithYearValueSmallerThen1400_ShouldThrowNewArgumentOutOfRangeException()
		{
			Executing.This(() =>
			{
				using (ISession session = TestSession)
				using (ITransaction tx = session.BeginTransaction())
				{
					var classUnderTest = new PostRepository(Factory, NullLogger.Instance);

					classUnderTest.GetByMonth(5, 1399, 1, 10);
				}
			})
				.Should()
				.Throw<ArgumentOutOfRangeException>()
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("year");
		}

		[Test]
		public void GetByMonth_WithValidParameters_ShouldReturnValidData()
		{
			var defaultCategory = this.AddDefaultCategoryToRepository();

			var testDate = new DateTime(2010, 1, 5, 05, 30, 30, DateTimeKind.Utc);

			//Add 3 elements in the current month
			this.AddNewPostToRepository("Title" + Guid.NewGuid(), Guid.NewGuid().ToString(), "abstract", "body", true, testDate, new[] { defaultCategory });
			this.AddNewPostToRepository("Title" + Guid.NewGuid(), Guid.NewGuid().ToString(), "abstract", "body", true, testDate.AddMinutes(5), new[] { defaultCategory });
			this.AddNewPostToRepository("Title" + Guid.NewGuid(), Guid.NewGuid().ToString(), "abstract", "body", true, testDate.AddMinutes(10), new[] { defaultCategory });

			//Add new element with the same month but different year. It shouldn't appear into the result
			this.AddNewPostToRepository("Title" + Guid.NewGuid(), Guid.NewGuid().ToString(), "abstract", "body", true, testDate.AddYears(1), new[] { defaultCategory });

			//Add 1 elements in the past month
			this.AddNewPostToRepository("Title" + Guid.NewGuid(), Guid.NewGuid().ToString(), "abstract", "body", true, testDate.AddMonths(-1), new[] { defaultCategory });

			using (ISession session = TestSession)
			using (ITransaction tx = session.BeginTransaction())
			{
				var sut = new PostRepository(Factory, NullLogger.Instance);

				var dataResult = sut.GetByMonth(testDate.Month, testDate.Year, 0, 2);

				dataResult.Should().Not.Be.Null();
				dataResult.Result.Should().Not.Be.Null();
				dataResult.Result.Count().Should().Be.EqualTo(2);
				dataResult.TotalCount.Should().Be.EqualTo(3);
			}
		}

		[Test]
		public void GetByMonth_WithUnpublished_ShouldReturnValidData()
		{
			var defaultCategory = this.AddDefaultCategoryToRepository();

			//Add 3 elements in the current month
			this.AddNewPostToRepository("Title" + Guid.NewGuid(), Guid.NewGuid().ToString(), "abstract", "body", true, DateTime.Today, new[] { defaultCategory });
			this.AddNewPostToRepository("Title" + Guid.NewGuid(), Guid.NewGuid().ToString(), "abstract", "body", false, DateTime.Today.AddMinutes(1), new[] { defaultCategory });
			this.AddNewPostToRepository("Title" + Guid.NewGuid(), Guid.NewGuid().ToString(), "abstract", "body", true, DateTime.Today.AddMinutes(5), new[] { defaultCategory });
			this.AddNewPostToRepository("Title" + Guid.NewGuid(), Guid.NewGuid().ToString(), "abstract", "body", true, DateTime.Today.AddMinutes(10), new[] { defaultCategory });
			this.AddNewPostToRepository("Title" + Guid.NewGuid(), Guid.NewGuid().ToString(), "abstract", "body", false, DateTime.Today.AddMinutes(15), new[] { defaultCategory });

			//Add 1 elements in the past month
			this.AddNewPostToRepository("Title" + Guid.NewGuid(), Guid.NewGuid().ToString(), "abstract", "body", true, DateTime.Today.AddMonths(-1), new[] { defaultCategory });

			using (ISession session = TestSession)
			using (ITransaction tx = session.BeginTransaction())
			{
				var sut = new PostRepository(Factory, NullLogger.Instance);

				var dataResult = sut.GetByMonth(DateTime.Now.Month, DateTime.Now.Year, 0, 2);

				tx.Commit();

				dataResult.Should().Not.Be.Null();
				dataResult.Result.Should().Not.Be.Null();
				dataResult.Result.Count().Should().Be.EqualTo(2);
				dataResult.Result.Any(x => x.IsPublished).Should().Be.True();
				dataResult.TotalCount.Should().Be.EqualTo(3);
			}
		}

		#endregion

		#region Method GetMostComment

		[Test]
		public void GetMostComment_WithNegativePageIndex_ShouldThrowArgumentOutOfRangeException()
		{
			Executing.This(() =>
			{
				using (ISession session = TestSession)
				using (ITransaction tx = session.BeginTransaction())
				{
					var classUnderTest = new PostRepository(Factory, NullLogger.Instance);

					classUnderTest.GetMostComment(-5, 10);
				}
			})
				.Should()
				.Throw<ArgumentOutOfRangeException>()
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("pageIndex");
		}

		[Test]
		public void GetMostComment_WithNegativePageSize_ShouldThrowNewArgumentOutOfRangeException()
		{
			Executing.This(() =>
			{
				using (ISession session = TestSession)
				using (ITransaction tx = session.BeginTransaction())
				{
					var classUnderTest = new PostRepository(Factory, NullLogger.Instance);

					classUnderTest.GetMostComment(1, -10);
				}
			})
				.Should()
				.Throw<ArgumentOutOfRangeException>()
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("pageSize");
		}

		[Test]
		public void GetMostComment_WithValidParametersAndEmptyRepository_ShouldNotReturnNull()
		{
			Assert.Inconclusive();

			using (ISession session = TestSession)
			using (ITransaction tx = session.BeginTransaction())
			{
				var sut = new PostRepository(Factory, NullLogger.Instance);

				var result = sut.GetMostComment(0, 10);

				result.Should().Not.Be.Null();
				result.TotalCount.Should().Be.EqualTo(0);
			}
		}

		[Test]
		public void GetMostComment_WithValidParameters_ShouldReturnValidData()
		{

			Assert.Inconclusive();

			var category = this.AddDefaultCategoryToRepository();
			var posts = this.AddNewPostsToRepository(new[] { category }, 4);

			posts[0].AddComments(true, 5);
			posts[1].AddComments(true, 2);

			this.Persist(posts[0]);
			this.Persist(posts[1]);

			using (ISession session = TestSession)
			using (ITransaction tx = session.BeginTransaction())
			{
				var sut = new PostRepository(Factory, NullLogger.Instance);

				var dataResult = sut.GetMostComment(1, 10);

				dataResult.Should().Not.Be.Null();
				dataResult.Result.Count().Should().Be.EqualTo(2);
				dataResult.Result.ToList()[0].Id.Should().Be.EqualTo(posts[0].Id);
				dataResult.Result.ToList()[1].Id.Should().Be.EqualTo(posts[1].Id);
			}

		}

		#endregion

		#region Method GetByCategory

		[Test]
		public void GetByCategory_WithNegativePageIndex_ShouldThrowArgumentOutOfRangeException()
		{
			Executing.This(() =>
			{
				using (ISession session = TestSession)
				using (ITransaction tx = session.BeginTransaction())
				{
					var classUnderTest = new PostRepository(Factory, NullLogger.Instance);

					classUnderTest.GetByCategory(1, -5, 10);
				}
			})
				.Should()
				.Throw<ArgumentOutOfRangeException>()
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("pageIndex");
		}

		[Test]
		public void GetByCategory_WithNegativePageSize_ShouldThrowNewArgumentOutOfRangeException()
		{
			Executing.This(() =>
			{
				using (ISession session = TestSession)
				using (ITransaction tx = session.BeginTransaction())
				{
					var classUnderTest = new PostRepository(Factory, NullLogger.Instance);

					classUnderTest.GetByCategory(1, 1, -10);
				}
			})
				.Should()
				.Throw<ArgumentOutOfRangeException>()
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("pageSize");
		}

		[Test]
		public void GetByCategory_WithZeroCategoryIdValue_ShouldThrowArgumentOutOfRangeException()
		{
			Executing.This(() =>
			{
				using (ISession session = TestSession)
				using (ITransaction tx = session.BeginTransaction())
				{
					var classUnderTest = new PostRepository(Factory, NullLogger.Instance);

					classUnderTest.GetByCategory(0, 5, 10);
				}
			})
				.Should()
				.Throw<ArgumentOutOfRangeException>()
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("categoryId");
		}

		[Test]
		public void GetByCategory_WithNegativeCategoryIdValue_ShouldThrowArgumentOutOfRangeException()
		{
			Executing.This(() =>
			{
				using (ISession session = TestSession)
				using (ITransaction tx = session.BeginTransaction())
				{
					var classUnderTest = new PostRepository(Factory, NullLogger.Instance);

					classUnderTest.GetByCategory(-5, 5, 10);
				}
			})
				.Should()
				.Throw<ArgumentOutOfRangeException>()
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("categoryId");
		}

		#endregion

		#region Method GetByAuthor

		[Test]
		public void GetByAuthor_WithNegativePageIndex_ShouldThrowArgumentOutOfRangeException()
		{
			Executing.This(() =>
			{
				using (ISession session = TestSession)
				using (ITransaction tx = session.BeginTransaction())
				{
					var classUnderTest = new PostRepository(Factory, NullLogger.Instance);

					classUnderTest.GetByAuthor("imperugo", -5, 10);
				}
			})
				.Should()
				.Throw<ArgumentOutOfRangeException>()
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("pageIndex");
		}

		[Test]
		public void GetByAuthor_WithNegativePageSize_ShouldThrowNewArgumentOutOfRangeException()
		{
			Executing.This(() =>
			{
				using (ISession session = TestSession)
				using (ITransaction tx = session.BeginTransaction())
				{
					var classUnderTest = new PostRepository(Factory, NullLogger.Instance);

					classUnderTest.GetByAuthor("imperugo", 1, -10);
				}
			})
				.Should()
				.Throw<ArgumentOutOfRangeException>()
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("pageSize");
		}

		[Test]
		public void GetByAuthor_WithNullAuthorValue_ShouldThrowNewThrowArgumentNullException()
		{
			Executing.This(() =>
			{
				using (ISession session = TestSession)
				using (ITransaction tx = session.BeginTransaction())
				{
					var classUnderTest = new PostRepository(Factory, NullLogger.Instance);

					classUnderTest.GetByAuthor(null, 1, 10);
				}
			})
				.Should()
				.Throw<ArgumentNullException>()
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("author");
		}

		[Test]
		public void GetByAuthor_WithEmptyAuthorValue_ShouldThrowNewThrowArgumentOutOfRangeException()
		{
			Executing.This(() =>
			{
				using (ISession session = TestSession)
				using (ITransaction tx = session.BeginTransaction())
				{
					var classUnderTest = new PostRepository(Factory, NullLogger.Instance);

					classUnderTest.GetByAuthor("", 1, 10);
				}
			})
				.Should()
				.Throw<ArgumentOutOfRangeException>()
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("author");
		}

		#endregion

		#region Method GetMonths

		[Test]
		public void GetMonths_WithNullUsername_ShouldReturnValidData()
		{
			var category = this.AddDefaultCategoryToRepository();

			//Adding past posts
			for (int i = 0; i < 3; i++)
			{
				this.AddNewPostToRepository("Title" + Guid.NewGuid(), Guid.NewGuid().ToString(), "abstract", "body", true, DateTime.Now.AddMonths(-1 * i), new[] { category });
			}

			//Adding futures posts
			this.AddNewPostToRepository("Title" + Guid.NewGuid(), Guid.NewGuid().ToString(), "abstract", "body", true, DateTime.Now.AddMonths(5), new[] { category });

			using (ISession session = TestSession)
			using (ITransaction tx = session.BeginTransaction())
			{
				var sut = new PostRepository(Factory, NullLogger.Instance);

				IEnumerable<Month> pagedResult = sut.GetMonths(null);

				pagedResult.Count().Should().Be.EqualTo(3);
			}
		}

		[Test]
		public void GetMonths_WithSpecificUsername_ShouldReturnValidData()
		{
			var category = this.AddDefaultCategoryToRepository();

			//Adding imperugo posts
			for (int i = 0; i < 3; i++)
			{
				var p = this.GetNewPost(new[] { category });
				p.PublishDate = DateTime.Today.AddMonths(-1 * i);
				p.Username = "imperugo";

				this.Persist(p);
			}

			//Adding imperugo2 posts
			var post = this.GetNewPost(new[] { category });
			post.Username = "imperugo2";
			this.Persist(post);

			using (ISession session = TestSession)
			using (ITransaction tx = session.BeginTransaction())
			{
				var sut = new PostRepository(Factory, NullLogger.Instance);

				IEnumerable<Month> result = sut.GetMonths("imperugo", true);

				result.Count().Should().Be.EqualTo(3);
			}
		}

		#endregion

		#region Method GetBySlug

		[Test]
		public void GetBySlug_WithMultipleCategories_ShouldReturnValidData()
		{
			var category = this.AddDefaultCategoryToRepository();
			var category2 = this.AddNewCategoryToRepository();

			//Adding imperugo2 posts
			var post = this.GetNewPost(new[] { category, category2 });
			string slug = Guid.NewGuid().ToString();
			post.Slug = slug;

			var tag1 = TagItem.CreateNewTagItem("tag1", post);
			var tag2 = TagItem.CreateNewTagItem("tag2", post);

			post.Tags.Add(tag1);
			post.Tags.Add(tag2);

			post.Tags.Count().Should().Be.EqualTo(2);

			Persist(post);

			using (ISession session = TestSession)
			using (ITransaction tx = session.BeginTransaction())
			{
				var sut = new PostRepository(Factory, NullLogger.Instance);

				var result = sut.GetBySlug(slug);

				result.Should().Not.Be.Null();
				result.Categories.Count().Should().Be.EqualTo(2);
				result.Tags.Count().Should().Be.EqualTo(2);

				tx.Commit();
			}
		}

		#endregion
	}
}
