﻿#region Disclaimer/Info

/////////////////////////////////////////////////////////////////////////////////////////////////
//
//   File:		TagRepositoryTest.cs
//   Website:		http://dexterblogengine.com/
//   Authors:		http://dexterblogengine.com/About.ashx
//   Rev:		1
//   Created:		28/12/2010
//   Last edit:		19/01/2011
//   License:		GNU Library General Public License (LGPL)
//   File:            TagRepositoryTest.cs
//   For updated news and information please visit http://dexterblogengine.com/
//   Dexter is hosted to Codeplex at http://dexterblogengine.codeplex.com
//   For any question contact info@dexterblogengine.com
//
///////////////////////////////////////////////////////////////////////////////////////////////////

#endregion

using System;
using System.Linq;
using Dexter.Domain.Model;
using Dexter.Logger.Concretes;
using Dexter.Logger.Contracts;
using Dexter.Repository.NHibernate.Repositories;
using NHibernate;
using NUnit.Framework;
using SharpTestsEx;

namespace Dexter.Repository.NHibernate.Test.Repositories
{
	[TestFixture]
	public class TagRepositoryTest : BaseNHibernateTest
	{
		public TagRepositoryTest() : base(true) { }

		[Test]
		public void GetItemsByTag_WithNegativePageIndex_ShouldThrowArgumentOutOfRangeException()
		{
			Executing.This(() =>
			{
				using (ISession session = TestSession)
				using (ITransaction tx = session.BeginTransaction())
				{
					var classUnderTest = new TagRepository(Factory, NullLogger.Instance);

					classUnderTest.GetItemsByTag(-5, 10, "tag");
				}

			})
				.Should()
				.Throw<ArgumentOutOfRangeException>()
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("pageIndex");
		}

		[Test]
		public void GetItemsByTag_WithNegativePageSize_ShouldThrowNewArgumentOutOfRangeException()
		{
			Executing.This(() =>
			{
				using (ISession session = TestSession)
				using (ITransaction tx = session.BeginTransaction())
				{
					var classUnderTest = new TagRepository(Factory, NullLogger.Instance);

					classUnderTest.GetItemsByTag(1, -10, "tag");
				}
			})
				.Should()
				.Throw<ArgumentOutOfRangeException>()
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("pageSize");
		}

		[Test]
		public void GetItemsByTag_WithNullTagname_ShouldThrowNewArgumentNullException()
		{
			Executing.This(() =>
			{
				using (ISession session = TestSession)
				using (ITransaction tx = session.BeginTransaction())
				{
					var classUnderTest = new TagRepository(Factory, NullLogger.Instance);

					classUnderTest.GetItemsByTag(1, 10, null);
				}
			})
				.Should()
				.Throw<ArgumentNullException>()
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("tagName");
		}

		[Test]
		public void GetItemsByTag_WithValidParametersAndEmptyRepository_ShouldNotReturnNull()
		{
			using (ISession session = TestSession)
			using (ITransaction tx = session.BeginTransaction())
			{
				var sut = new TagRepository(Factory, NullLogger.Instance);

				var result = sut.GetItemsByTag(1, 10, "tag");

				result.Should().Not.Be.Null();
				result.TotalCount.Should().Be.EqualTo(0);
			}
		}

		[Test]
		public void GetItemsByTag_WithValidParameters_ShouldReturnValidaData()
		{
			//Assert.Inconclusive();

			var defaultCategory = this.AddDefaultCategoryToRepository();
			var post = this.GetNewPost(new[] { defaultCategory });
			var post2 = this.GetNewPost(new[] { defaultCategory });
			var post3 = this.GetNewPost(new[] { defaultCategory });

			post.Tags.Add(new TagItem("tag1", post));
			post.Tags.Add(new TagItem("tag2", post));
			post.Tags.Add(new TagItem("tag3", post));
			post.Tags.Add(new TagItem("tag4", post));

			post2.Tags.Add(new TagItem("tag1", post2));
			post2.Tags.Add(new TagItem("tag2", post2));

			post3.Tags.Add(new TagItem("tag3", post3));
			post3.Tags.Add(new TagItem("tag4", post3));

			this.Persist(post, post2, post3);

			using (ISession session = TestSession)
			using (ITransaction tx = session.BeginTransaction())
			{
				var sut = new TagRepository(Factory, NullLogger.Instance);

				var result = sut.GetItemsByTag(0, 1, "tag1");

				result.Should().Not.Be.Null();
				result.TotalCount.Should().Be.EqualTo(2);
				result.Result.Count().Should().Be.EqualTo(1);
			}
		}

		[Test]
		public void GetTopTags_WithParameter_ShouldThrowArgumentOutOfRangeException()
		{
			Executing.This(() =>
			{
				using (ISession session = TestSession)
				using (ITransaction tx = session.BeginTransaction())
				{
					var classUnderTest = new TagRepository(Factory, NullLogger.Instance);

					classUnderTest.GetTopTags(-5);
				}
			})
				.Should()
				.Throw<ArgumentOutOfRangeException>()
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("numberOfItemsToRetireve");
		}

		[Test]
		public void GetTopTags_WithValidParameters_ShouldReturnValidaData()
		{
			//Assert.Inconclusive();

			var defaultCategory = this.AddDefaultCategoryToRepository();
			var post = this.GetNewPost(new[] { defaultCategory });
			var post2 = this.GetNewPost(new[] { defaultCategory });
			var post3 = this.GetNewPost(new[] { defaultCategory });

			post.Tags.Add(new TagItem("tag1", post));
			post.Tags.Add(new TagItem("tag2", post));
			post.Tags.Add(new TagItem("tag3", post));
			post.Tags.Add(new TagItem("tag4", post));

			post2.Tags.Add(new TagItem("tag1", post2));
			post2.Tags.Add(new TagItem("tag2", post2));

			post3.Tags.Add(new TagItem("tag3", post3));
			post3.Tags.Add(new TagItem("tag4", post3));

			this.Persist(post, post2, post3);

			using (ISession session = TestSession)
			using (ITransaction tx = session.BeginTransaction())
			{
				var sut = new TagRepository(Factory, NullLogger.Instance);

				var result = sut.GetTopTags(2);

				result.Should().Not.Be.Null();
				result.Count().Should().Be.EqualTo(2);
				result.ToList()[0].Name.Should().Be.EqualTo("tag1");
				result.ToList()[1].Name.Should().Be.EqualTo("tag2");
			}
		}

		[Test]
		public void GetPostsByTag_WithValidData_ShouldReturnTheCorrectResult()
		{
			var defaultCategory = this.AddDefaultCategoryToRepository();
			var post = this.GetNewPost(new[] { defaultCategory });
			var post2 = this.GetNewPost(new[] { defaultCategory });
			var post3 = this.GetNewPost(new[] { defaultCategory });

			post.Tags.Add(new TagItem("tag1", post));
			post.Tags.Add(new TagItem("tag2", post));
			post.Tags.Add(new TagItem("tag3", post));
			post.Tags.Add(new TagItem("tag4", post));

			post2.Tags.Add(new TagItem("tag1", post2));
			post2.Tags.Add(new TagItem("tag2", post2));

			post3.Tags.Add(new TagItem("tag3", post3));
			post3.Tags.Add(new TagItem("tag4", post3));

			this.Persist(post, post2, post3);

			using (ISession session = TestSession)
			using (ITransaction tx = session.BeginTransaction())
			{
				var sut = new TagRepository(Factory, NullLogger.Instance);

				var data = sut.GetPostsByTag(0, 10, "tag1");

				data.Should().Not.Be.Null();
				data.TotalCount.Should().Be.EqualTo(2);
			}
		}
	}
}
