﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using NUnit.Framework;
using SharpTestsEx;

namespace Dexter.Storage.FileSystem.Test {
	[TestFixture]
	public class FileSystemStorageProviderTests {
		string filePath;
		string folderPath;
		IStorageProvider storageProvider;

		[SetUp]
		public void Init ( ) {
			folderPath = Path.Combine ( AppDomain.CurrentDomain.BaseDirectory , "Media" );
			filePath = folderPath + "\\testfile.txt";

			if ( Directory.Exists ( folderPath ) ) {
				Directory.Delete ( folderPath , true );
			}

			Directory.CreateDirectory ( folderPath );
			File.WriteAllText ( filePath , "testfile contents" );

			var subfolder1 = Path.Combine ( folderPath , "Subfolder1" );
			Directory.CreateDirectory ( subfolder1 );
			File.WriteAllText ( Path.Combine ( subfolder1 , "one.txt" ) , "one contents" );
			File.WriteAllText ( Path.Combine ( subfolder1 , "two.txt" ) , "two contents" );

			var subsubfolder1 = Path.Combine ( subfolder1 , "SubSubfolder1" );
			Directory.CreateDirectory ( subsubfolder1 );

			storageProvider = new FileSystemStorageProvider ( "Media", "App_Data");
		}

		[TearDown]
		public void Term ( ) {
			Directory.Delete ( folderPath , true );
		}

		[Test]
		public void GetFile_WithInvalidFilePath_ShouldThrowFileNotFoundException ( ) {
			Executing.This(() => storageProvider.GetFile("notexistingpath"))
				.Should()
				.Throw<FileNotFoundException>();
		}

		[Test]
		public void ListFiles_WithValidRepository_ShouldReturnFilesFromFilesystem ( ) {
			IEnumerable <IStorageFile> files = storageProvider.ListFiles ( folderPath );
			files.Count ( ).Should ( ).Be.EqualTo ( 1 );
		}

		[Test]
		public void GetFile_WithValidRepository_ShouldReturnShortPath ( ) {
			var filePath = storageProvider.Combine(storageProvider.PublicRoot, "testfile.txt");
			var file = storageProvider.GetFile(filePath);

			file.Should ( ).Not.Be.Null ( );
			file.GetPath().Should().Be.EqualTo(filePath);
			file.GetName ( ).Should ( ).Be.EqualTo ( "testfile.txt" );
		}

		[Test]
		public void ListFilesReturnsItemsWithShortPathAndEnvironmentSlashes ( ) {
			var folderPath = storageProvider.Combine(storageProvider.PublicRoot, "Subfolder1");
			var files = storageProvider.ListFiles(folderPath);

			files.Should ( ).Not.Be.Null ( );
			files.Count ( ).Should ( ).Be.EqualTo ( 2 );

			var one = files.Single ( x => x.GetName ( ) == "one.txt" );
			var two = files.Single ( x => x.GetName ( ) == "two.txt" );

			one.GetPath ( ).Should ( ).Be.EqualTo ( "Media\\Subfolder1" + Path.DirectorySeparatorChar + "one.txt" );
			two.GetPath().Should().Be.EqualTo("Media\\Subfolder1" + Path.DirectorySeparatorChar + "two.txt");
		}

		[Test]
		public void AnySlashInGetFileBecomesEnvironmentAppropriate ( ) {
			var file1 = storageProvider.GetFile ( @"Media/Subfolder1/one.txt" );
			var file2 = storageProvider.GetFile ( @"Media\Subfolder1\one.txt" );

			file1.GetPath ( ).Should ( ).Be.EqualTo ( "Media\\Subfolder1" + Path.DirectorySeparatorChar + "one.txt" );
			file2.GetPath().Should().Be.EqualTo("Media\\Subfolder1" + Path.DirectorySeparatorChar + "one.txt");
		}

		[Test]
		public void ListFoldersReturnsItemsWithShortPathAndEnvironmentSlashes ( ) {
			var folders = storageProvider.ListFolders ( @"Media\Subfolder1" );

			folders.Should ( ).Not.Be.Null ( );
			folders.Count ( ).Should ( ).Be.EqualTo ( 1 );
			folders.Single().GetName().Should().Be.EqualTo("SubSubfolder1");
			folders.Single().GetPath().Should().Be.EqualTo(Path.Combine("Media\\Subfolder1", "SubSubfolder1"));
		}

		[Test]
		public void ParentFolderPathIsStillShort ( ) {
			var subsubfolder = storageProvider.ListFolders ( @"Media\Subfolder1" ).Single ( );
			var subfolder = subsubfolder.GetParent ( );

			subsubfolder.GetName ( ).Should ( ).Be.EqualTo ( "SubSubfolder1" );
			subsubfolder.GetPath ( ).Should ( ).Be.EqualTo ( Path.Combine ( "Media\\Subfolder1" , "SubSubfolder1" ) );
			subfolder.GetName ( ).Should ( ).Be.EqualTo ( "Subfolder1" );
			subfolder.GetPath ( ).Should ( ).Be.EqualTo ( "Media\\Subfolder1" );
		}

		[Test]
		public void CreateFolderAndDeleteFolderTakesAnySlash ( ) {
			storageProvider.ListFolders ( @"Media\\Subfolder1" ).Count ( ).Should ( ).Be.EqualTo ( 1 );

			storageProvider.CreateFolder ( @"Media/SubFolder1/SubSubFolder2" );
			storageProvider.CreateFolder ( @"Media\SubFolder1\SubSubFolder3" );

			storageProvider.ListFolders ( @"Media\Subfolder1" ).Count ( ).Should ( ).Be.EqualTo ( 3 );

			storageProvider.DeleteFolder ( @"Media/SubFolder1/SubSubFolder2" );
			storageProvider.DeleteFolder ( @"Media\SubFolder1\SubSubFolder3" );

			storageProvider.ListFolders ( @"Media\Subfolder1" ).Count ( ).Should ( ).Be.EqualTo ( 1 );
		}

		[Test]
		public void RenameFolderTakesShortPathWithAnyKindOfSlash ( ) {
			StorageHelper.GetFolder (storageProvider, @"Media/SubFolder1/SubSubFolder1" ).Should ( ).Not.Be.Null ( );

			storageProvider.RenameFolder ( @"Media\SubFolder1\SubSubFolder1" , @"Media/SubFolder1/SubSubFolder2" );
			storageProvider.RenameFolder ( @"Media\SubFolder1\SubSubFolder2" , @"Media/SubFolder1\SubSubFolder3" );
			storageProvider.RenameFolder ( @"Media\SubFolder1/SubSubFolder3" , @"Media/SubFolder1\SubSubFolder4" );
			storageProvider.RenameFolder ( @"Media\SubFolder1/SubSubFolder4" , @"Media/SubFolder1/SubSubFolder5" );

			StorageHelper.GetFolder(storageProvider, Path.Combine("Media\\SubFolder1", "SubSubFolder1")).Should().Be.Null();
			StorageHelper.GetFolder(storageProvider, Path.Combine("Media\\SubFolder1", "SubSubFolder2")).Should().Be.Null();
			StorageHelper.GetFolder(storageProvider, Path.Combine("Media\\SubFolder1", "SubSubFolder3")).Should().Be.Null();
			StorageHelper.GetFolder(storageProvider, Path.Combine("Media\\SubFolder1", "SubSubFolder4")).Should().Be.Null();
			StorageHelper.GetFolder(storageProvider, Path.Combine("Media\\SubFolder1", "SubSubFolder5")).Should().Not.Be.Null();
		}

		[Test]
		public void CreateFileAndDeleteFileTakesAnySlash ( ) {
			storageProvider.ListFiles ( @"Media\Subfolder1" ).Count ( ).Should ( ).Be.EqualTo ( 2 );

			var alpha = storageProvider.CreateFile ( @"Media/SubFolder1/alpha.txt" );
			var beta = storageProvider.CreateFile ( @"Media\SubFolder1\beta.txt" );

			storageProvider.ListFiles ( @"Media\Subfolder1" ).Count ( ).Should ( ).Be.EqualTo ( 4 );
			alpha.GetPath ( ).Should ( ).Be.EqualTo ( Path.Combine ( "Media\\SubFolder1" , "alpha.txt" ) );
			beta.GetPath ( ).Should ( ).Be.EqualTo ( Path.Combine ( "Media\\SubFolder1" , "beta.txt" ) );

			storageProvider.DeleteFile ( @"Media\SubFolder1\alpha.txt" );
			storageProvider.DeleteFile ( @"Media/SubFolder1/beta.txt" );

			storageProvider.ListFiles ( @"Media\Subfolder1" ).Count ( ).Should ( ).Be.EqualTo ( 2 );
		}

		[Test]
		public void RenameFileTakesShortPathWithAnyKindOfSlash ( ) {

			StorageHelper.GetFile ( storageProvider, @"Subfolder1/one.txt" ).Should ( ).Not.Be.Null ( );

			storageProvider.RenameFile ( @"SubFolder1\one.txt" , @"SubFolder1/testfile2.txt" );
			storageProvider.RenameFile ( @"SubFolder1\testfile2.txt" , @"SubFolder1\testfile3.txt" );
			storageProvider.RenameFile ( @"SubFolder1/testfile3.txt" , @"SubFolder1\testfile4.txt" );
			storageProvider.RenameFile ( @"SubFolder1/testfile4.txt" , @"SubFolder1/testfile5.txt" );

			StorageHelper.GetFile ( storageProvider, Path.Combine ( "SubFolder1", "one.txt" ) ).Should ( ).Be.Null ( );
			StorageHelper.GetFile ( storageProvider, Path.Combine ( "SubFolder1", "testfile2.txt" ) ).Should ( ).Be.Null ( );
			StorageHelper.GetFile ( storageProvider, Path.Combine ( "SubFolder1", "testfile3.txt" ) ).Should ( ).Be.Null ( );
			StorageHelper.GetFile ( storageProvider, Path.Combine ( "SubFolder1", "testfile4.txt" ) ).Should ( ).Be.Null ( );
			StorageHelper.GetFile ( storageProvider, Path.Combine ( "SubFolder1", "testfile5.txt" ) ).Should ( ).Not.Be.Null ( );
		}

		[Test]
		public void GetFileFailsInInvalidPath ( ) {
			Executing.This ( ( ) => storageProvider.GetFile ( @"../InvalidFile.txt" ) )
				.Should ( )
				.Throw <ArgumentException> ( );

			Executing.This ( ( ) => storageProvider.GetFile ( @"../../InvalidFile.txt" ) )
				.Should ( )
				.Throw <ArgumentException> ( );

			// Valid get one level up within the storage provider domain
			storageProvider.CreateFile ( @"test.txt" );

			storageProvider.GetFile ( @"test.txt" ).Should ( ).Not.Be.Null ( );
			storageProvider.GetFile ( @"SubFolder1\..\test.txt" ).Should ( ).Not.Be.Null ( );
		}

		[Test]
		public void ListFilesFailsInInvalidPath ( ) {
			Executing.This ( ( ) => storageProvider.ListFiles ( @"../InvalidFolder" ) )
				.Should ( )
				.Throw <ArgumentException> ( );

			Executing.This ( ( ) => storageProvider.ListFiles ( @"../../InvalidFolder" ) )
				.Should ( )
				.Throw <ArgumentException> ( );

			storageProvider.ListFiles ( @"SubFolder1" ).Should ( ).Not.Be.Null ( );
			storageProvider.ListFiles ( @"SubFolder1\.." ).Should ( ).Not.Be.Null ( );
		}

		[Test]
		public void ListFoldersFailsInInvalidPath ( ) {
			Executing.This ( ( ) => storageProvider.ListFolders ( @"../InvalidFolder" ) )
				.Should ( )
				.Throw <ArgumentException> ( );

			Executing.This ( ( ) => storageProvider.ListFolders ( @"../../InvalidFolder" ) )
				.Should ( )
				.Throw <ArgumentException> ( );

			storageProvider.ListFolders ( @"SubFolder1" ).Should ( ).Not.Be.Null ( );
			storageProvider.ListFolders ( @"SubFolder1\.." ).Should ( ).Not.Be.Null ( );
		}

		[Test]
		public void TryCreateFolderFailsInInvalidPath ( ) {
			storageProvider.TryCreateFolder ( @"../InvalidFolder1" ).Should ( ).Be.False ( );
			storageProvider.TryCreateFolder ( @"../../InvalidFolder1" ).Should ( ).Be.False ( );

			// Valid create one level up within the storage provider domain
			storageProvider.TryCreateFolder ( @"SubFolder1\..\ValidFolder1" ).Should ( ).Be.True ( );
		}

		[Test]
		public void CreateFolderFailsInInvalidPath ( ) {
			Executing.This ( ( ) => storageProvider.CreateFolder ( @"../InvalidFolder1" ) )
				.Should ( )
				.Throw <ArgumentException> ( );

			Executing.This ( ( ) => storageProvider.CreateFolder ( @"../../InvalidFolder1" ) )
				.Should ( )
				.Throw <ArgumentException> ( );

			// Valid create one level up within the storage provider domain
			storageProvider.CreateFolder ( @"SubFolder1\..\ValidFolder1" );

			StorageHelper.GetFolder (storageProvider, "ValidFolder1" ).Should ( ).Not.Be.Null ( );
		}

		[Test]
		public void DeleteFolderFailsInInvalidPath ( ) {
			Executing.This ( ( ) => storageProvider.DeleteFolder ( @"Media\../InvalidFolder1" ) )
				.Should ( )
				.Throw <ArgumentException> ( );

			Executing.This ( ( ) => storageProvider.DeleteFolder ( @"Media\../../InvalidFolder1" ) )
				.Should ( )
				.Throw <ArgumentException> ( );


			// Valid create one level up within the storage provider domain
			StorageHelper.GetFolder ( storageProvider, "Media\\SubFolder1" ).Should ( ).Not.Be.Null ( );
			storageProvider.DeleteFolder(@"Media\SubFolder1\..\SubFolder1");
			StorageHelper.GetFolder(storageProvider, "Media\\SubFolder1").Should().Be.Null();
		}

		[Test]
		public void RenameFolderFailsInInvalidPath ( ) {
			StorageHelper.GetFolder ( storageProvider, @"SubFolder1/SubSubFolder1" ).Should ( ).Not.Be.Null ( );

			Executing.This ( ( ) => storageProvider.RenameFolder ( @"SubFolder1" , @"../SubSubFolder1" ) )
				.Should ( )
				.Throw <ArgumentException> ( );

			Executing.This ( ( ) => storageProvider.RenameFolder ( @"SubFolder1" , @"../../SubSubFolder1" ) )
				.Should ( )
				.Throw <ArgumentException> ( );

			// Valid move one level up within the storage provider domain
			storageProvider.RenameFolder ( @"SubFolder1\SubSubFolder1" , @"SubFolder1\..\SubSubFolder1" );
			StorageHelper.GetFolder ( storageProvider, "SubSubFolder1" ).Should ( ).Not.Be.Null ( );

			storageProvider.CreateFolder ( @"SubFolder1\SubSubFolder1\SubSubSubFolder1" );
			storageProvider.RenameFolder ( @"SubFolder1\SubSubFolder1\SubSubSubFolder1" , @"SubFolder1\SubSubFolder1\..\SubSubSubFolder1" );

			StorageHelper.GetFolder ( storageProvider, @"SubFolder1\SubSubSubFolder1" ).Should ( ).Not.Be.Null ( );
		}

		[Test]
		public void DeleteFileFailsInInvalidPath ( ) {
			Executing.This ( ( ) => storageProvider.DeleteFile ( @"../test.txt" ) )
				.Should ( )
				.Throw <ArgumentException> ( );

			// Valid move one level up within the storage provider domain
			storageProvider.CreateFile ( @"Media\test.txt" );

			StorageHelper.GetFile(storageProvider, "Media\\test.txt").Should().Not.Be.Null();

			storageProvider.DeleteFile(@"Media\test.txt");
			StorageHelper.GetFile(storageProvider, "Media\\test.txt").Should().Be.Null();

			storageProvider.CreateFile(@"Media\test.txt");
			StorageHelper.GetFile(storageProvider, "Media\\test.txt").Should().Not.Be.Null();
			storageProvider.DeleteFile(@"Media\SubFolder1\..\test.txt");
			StorageHelper.GetFile(storageProvider, "Media\\test.txt").Should().Be.Null();
		}

		[Test]
		public void RenameFileFailsInInvalidPath ( ) {
			Executing.This ( ( ) => storageProvider.RenameFile ( @"../test.txt" , "invalid.txt" ) )
				.Should ( )
				.Throw <ArgumentException> ( );

			// Valid move one level up within the storage provider domain
			storageProvider.CreateFile ( @"test.txt" );
			StorageHelper.GetFile ( storageProvider, "test.txt" ).Should ( ).Not.Be.Null ( );

			storageProvider.RenameFile ( @"test.txt" , "newName.txt" );
			StorageHelper.GetFile ( storageProvider, "newName.txt" ).Should ( ).Not.Be.Null ( );

			storageProvider.RenameFile ( @"SubFolder1\..\newName.txt" , "newNewName.txt" );
			StorageHelper.GetFile ( storageProvider, "newNewName.txt" ).Should ( ).Not.Be.Null ( );
		}

		[Test]
		public void CreateFileFailsInInvalidPath ( ) {
			Executing.This ( ( ) => storageProvider.CreateFile ( @"InvalidFolder1.txt" ) )
				.Should ( )
				.Throw <ArgumentException> ( );

			Executing.This ( ( ) => storageProvider.CreateFile ( @"../InvalidFolder1.txt" ) )
				.Should ( )
				.Throw <ArgumentException> ( );


			// Valid create one level up within the storage provider domain
			storageProvider.CreateFile ( @"Media\SubFolder1\..\ValidFolder1.txt" );
			StorageHelper.GetFile ( storageProvider, "Media\\ValidFolder1.txt" ).Should ( ).Not.Be.Null ( );
		}

		[Test]
		public void SaveStreamFailsInInvalidPath ( ) {
			storageProvider.CreateFile ( @"test.txt" );

			using (Stream stream = StorageHelper.GetFile ( storageProvider, "test.txt" ).OpenRead ( )) {
				Executing.This ( ( ) => storageProvider.SaveStream ( @"../newTest.txt" , stream ) )
					.Should ( )
					.Throw <ArgumentException> ( );

				Executing.This ( ( ) => storageProvider.SaveStream ( @"../../newTest.txt" , stream ) )
					.Should ( )
					.Throw <ArgumentException> ( );

				// Valid create one level up within the storage provider domain
				storageProvider.SaveStream ( @"SubFolder1\..\newTest.txt" , stream );
				StorageHelper.GetFile ( storageProvider, "newTest.txt" ).Should ( ).Not.Be.Null ( );
			}
		}

		[Test]
		public void TrySaveStreamFailsInInvalidPath ( ) {
			storageProvider.CreateFile ( @"test.txt" );

			using (Stream stream = StorageHelper.GetFile ( storageProvider, "test.txt" ).OpenRead ( )) {
				storageProvider.TrySaveStream ( @"../newTest.txt" , stream ).Should ( ).Be.False ( );
				storageProvider.TrySaveStream ( @"../../newTest.txt" , stream ).Should ( ).Be.False ( );


				// Valid create one level up within the storage provider domain
				storageProvider.TrySaveStream ( @"SubFolder1\..\newTest.txt" , stream ).Should ( ).Be.True ( );
			}
		}

		[Test]
		public void GetFolder_WithValidPath_ShouldReturnValidStorageFolder() {
			var result = storageProvider.GetFolder ( "Subfolder1" );

			result.Should ( ).Not.Be.Null ( );
		}

		[Test]
		public void FolderExists_WithValidPath_ShouldReturnValidStorageFolder ( ) {
			var result = storageProvider.FolderExists ( "Subfolder1" );

			result.Should ( ).Be.True (  );
		}
	}
}